/**
 *  @file  <calcomponent.h> 
 *  @brief functions to access to component, property and parameter
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */
#ifndef _LIPS_CAL_COMPONENT_H
#define _LIPS_CAL_COMPONENT_H

#include  "caltype.h"
#include  "calerror.h"


#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

/**
 * @brief  Refer to RFC2445 4.6 : Calendar Components
 *  Types of  components to be stored by a cal_component_t.
 */
  typedef enum
  {
    CAL_NO_TYPE_COMPONENT = 0,
    CAL_EVENT_COMPONENT,
    CAL_TODO_COMPONENT,
    CAL_ALARM_COMPONENT,
    CAL_JOURNAL_COMPONENT,
    CAL_FREEBUSY_COMPONENT,
    CAL_TIMEZONE_COMPONENT
  } cal_comp_type_t;


  
/** 
 * @brief property value type 
 */
  typedef enum
  {
    CAL_PROP_VALUE_TYPE_INT32 = 0,
		CAL_PROP_VALUE_TYPE_INT64,
    CAL_PROP_VALUE_TYPE_FLOAT,
		CAL_PROP_VALUE_TYPE_DOUBLE,
    CAL_PROP_VALUE_TYPE_STRING,
    CAL_PROP_VALUE_TYPE_BINARY
  } cal_prop_value_type_t;

  
/**
* @brief parameter type
*/
  typedef enum
  {
    CAL_ANY_PARAMETER = 0,
    CAL_ACTIONPARAM_PARAMETER,  /**<RFC2445 4.8.6.1 */
    CAL_ALTREP_PARAMETER,       /**<RFC2445 4.2.1 */
    CAL_CN_PARAMETER,           /**<RFC2445 4.2.2 */
    CAL_CUTYPE_PARAMETER,       /**<RFC2445 4.2.3 */
    CAL_DELEGATEDFROM_PARAMETER, /**<RFC2445 4.2.4 */
    CAL_DELEGATEDTO_PARAMETER,  /**<RFC2445 4.2.5 */
    CAL_DIR_PARAMETER,          /**<RFC2445 4.2.6 */
		CAL_ENABLE_PARAMETER,
		CAL_ENCODING_PARAMETER,     /**<RFC2445 4.2.7 */
    CAL_FBTYPE_PARAMETER,       /**<RFC2445 4.2.9 */
    CAL_FMTTYPE_PARAMETER,      /**<RFC2445 4.2.8 */
		CAL_ID_PARAMETER,		
    CAL_LANGUAGE_PARAMETER,     /**<RFC2445 4.2.10 */
		CAL_LATENCY_PARAMETER,
		CAL_LOCAL_PARAMETER,
    CAL_LOCALIZE_PARAMETER,
    CAL_MEMBER_PARAMETER,       /**<RFC2445 4.2.11 */
		CAL_OPTIONS_PARAMETER,
    CAL_PARTSTAT_PARAMETER,     /**<RFC2445 4.2.12 */
    CAL_RANGE_PARAMETER,        /**<RFC2445 4.2.13 */
    CAL_RELATED_PARAMETER,      /**<RFC2445 4.2.14 */  
    CAL_RELTYPE_PARAMETER,      /**<RFC2445 4.2.15 */
    CAL_ROLE_PARAMETER,         /**<RFC2445 4.2.16 */
    CAL_RSVP_PARAMETER,         /**<RFC2445 4.2.17 */
    CAL_SENTBY_PARAMETER,       /**<RFC2445 4.2.18 */
    CAL_TZID_PARAMETER,         /**<RFC2445 4.2.19 */
    CAL_VALUE_PARAMETER,        /**<RFC2445 4.2.20 */
    CAL_X_PARAMETER,             /**<Other parameter */
    CAL_INVALID_PARAMETER
  } cal_standard_parameter_type_t;



/**
* @brief property type
*/  
  typedef enum
  {
    CAL_ANY_PROPERTY = 0,
    CAL_ATTACHURI_PROPERTY,           /**<RFC2445 4.8.1.1 descriptive component property*/
  	CAL_ATTACHINLINE_PROPERTY,
	  CAL_ATTENDEE_PROPERTY,         /**<RFC2445 4.8.4.1 relationship component property*/
    /*CAL_CALID_PROPERTY,*/       /**<RFC2445 4.8.4.7 relationship component property*/
    /*CAL_CALSCALE_PROPERTY,*/    /**<RFC2445 4.7.1 calendar property*/
    CAL_CATEGORIES_PROPERTY,      /**<RFC2445 4.8.1.2 descriptive component property*/ 
  	CAL_CHANNEL_PROPERTY,
	  CAL_CLASS_PROPERTY,            /**<RFC2445 4.8.1.3 descriptive component property*/ 
    CAL_COMMENT_PROPERTY,          /**<RFC2445 4.8.1.4 descriptive component property*/ 
    CAL_COMPLETED_PROPERTY,        /**<RFC2445 4.8.2.1 date and time component property*/ 
    CAL_CONTACT_PROPERTY,          /**<RFC2445 4.8.4.2 relationship component property*/ 
    CAL_CREATED_PROPERTY,          /**<RFC2445 4.8.7.1 change management component property*/ 
    CAL_DESCRIPTION_PROPERTY,     /**<RFC2445 4.8.1.5 descriptive component property*/
    CAL_DTEND_PROPERTY,            /**<RFC2445 4.8.2.2 date and time component property*/
    CAL_DTSTAMP_PROPERTY,          /**<RFC2445 4.8.7.2 change management component property*/ 
    CAL_DTSTART_PROPERTY,         /**<RFC2445 4.8.2.4 date and time component property*/
    CAL_DUE_PROPERTY,              /**<RFC2445 4.8.2.3 date and time component property*/
    CAL_DURATION_PROPERTY,         /**<RFC2445 4.8.2.5 date and time component property*/
    CAL_EXDATE_PROPERTY,           /**<RFC2445 4.8.5.1 recurrence component property*/
    CAL_EXRULE_PROPERTY,           /**<RFC2445 4.8.5.2 recurrence component property*/
    CAL_FREEBUSY_PROPERTY,         /**<RFC2445 4.8.2.6 date and time component property*/
    CAL_GEO_PROPERTY,             /**<RFC2445 4.8.1.6 descriptive component property*/
    CAL_LASTMODIFIED_PROPERTY,     /**<RFC2445 4.8.7.3 change management component property*/
    CAL_LOCATION_PROPERTY,         /**<RFC2445 4.8.1.7 descriptive component property*/
    CAL_METHOD_PROPERTY,      /**<RFC2445 4.7.2 calendar property*/
    CAL_ORGANIZER_PROPERTY,       /**<RFC2445 4.8.4.3 relationship component property*/
    CAL_PERCENTCOMPLETE_PROPERTY, /**<RFC2445 4.8.1.8 descriptive component property*/
    CAL_PRIORITY_PROPERTY,         /**<RFC2445 4.8.1.9 descriptive component property*/
    /*CAL_PRODID_PROPERTY,*/      /**<RFC2445 4.7.3 calendar property*/
    CAL_RDATE_PROPERTY,            /**<RFC2445 4.8.5.3 recurrence component property*/
    CAL_RECURRENCEID_PROPERTY,     /**<RFC2445 4.8.4.4 relationship component property*/
    CAL_RELATEDTO_PROPERTY,       /**<RFC2445 4.8.4.5 relationship component property*/
    CAL_REQUESTSTATUS_PROPERTY,   /**<RFC2445 4.8.8.2 miscellaneous component property*/
    CAL_RESOURCES_PROPERTY,       /**<RFC2445 4.8.1.10 descriptive component property*/
    CAL_RRULE_PROPERTY,            /**<RFC2445 4.8.5.4 recurrence component property*/ 
    CAL_SEQUENCE_PROPERTY,         /**<RFC2445 4.8.7.4 change management component property*/
    CAL_STATUS_PROPERTY,           /**<RFC2445 4.8.1.11 descriptive component property*/
    CAL_SUMMARY_PROPERTY,         /**<RFC2445 4.8.1.12 descriptive component property*/
    CAL_TRANSP_PROPERTY,           /**<RFC2445 4.8.2.7 date and time component property*/
    CAL_TZID_PROPERTY,             /**<RFC2445 4.8.3.1 time zone component property*/
    CAL_TZNAME_PROPERTY,           /**<RFC2445 4.8.3.2 time zone component property*/
    CAL_TZOFFSETFROM_PROPERTY,     /**<RFC2445 4.8.3.3 time zone component property*/
    CAL_TZOFFSETTO_PROPERTY,       /**<RFC2445 4.8.3.4 time zone component property*/
    CAL_TZURL_PROPERTY,           /**<RFC2445 4.8.3.5 time zone component property*/
    CAL_UID_PROPERTY,             /**<RFC2445 4.8.4.7 relationship component property*/
    CAL_ALARM_ACTION_PROPERTY,           /**<RFC2445 4.8.6.1 alarm component property*/
    CAL_ALARM_REPEAT_PROPERTY,           /**<RFC2445 4.8.6.2 alarm component property*/
    CAL_ALARM_TRIGGER_PROPERTY,         /**<RFC2445 4.8.6.3 alarm component property*/
    CAL_ALARM_DURATION_PROPERTY,         /**<RFC2445 4.8.2.5 date and time component property*/
    CAL_ALARM_DESCRIPTION_PROPERTY,     /**<RFC2445 4.8.1.5 descriptive component property*/
    CAL_ALARM_ATTACH_PROPERTY,           /**<RFC2445 4.8.1.1 descriptive component property*/
    CAL_ALARM_ATTENDEE_PROPERTY         /**<RFC2445 4.8.4.1 relationship component property*/
   } cal_standard_prop_type_t;


/**
 *  @struct cal_component_t   
 *  @brief an opaque structure to hold a component.
 *   The instance structure _cal_component_t must be defined in
 *   implementation source code.
 */
  typedef struct _cal_component_t cal_component_t;


/*********************************************************************** 
 * create, copy, convert, and destroy of the cal_component_t structure *
 ***********************************************************************/

/** @brief     create a new component.
*
* @param[in]	   sid         session id 
* @param[in]     type      the type of the component. 
* @param[out]   comp     pointer to the memory location where a newly 
*                        created component will be returned. 
*
* @return   CAL_ERR_NONE            if successfull
*           CAL_ERR_*               other errors.
*
* @note     applications should later free the component using the 
*           cal_component_free() call.
*/
  cal_err_t cal_component_new (cal_sid_t sid, cal_comp_type_t type, 
  																 cal_component_t **comp);


/** @brief     free a component. 
*
* @param[in]   comp          a pointer to the component.
*
* @return   CAL_ERR_NONE            if successfully
*           CAL_ERR_*               other errors.
*/
  cal_err_t cal_component_free (cal_component_t *comp);

/** @brief     free an array of components. 
*
* This function will free not only the pointer to the list of components,
* but also the actual elements.(Consider in the implementation:If the 
* component is detached component, this function will not free the elements.)
*
* @param[in]    comps       a pointer to an array of component pointers, terminated with a NULL pointer.
*
* @return   CAL_ERR_NONE            if successful
*           CAL_ERR_*               other errors.
*/
  cal_err_t cal_components_free (cal_component_t **comps);

/** @brief    get the type of the component.
*
*  The type can only be specified when creating a new component, 
*  it can't be modified after the component has been created.
*
* @param[in]   comp           a pointer to the component.
* @param[out]  type           the component's type.
*
* @return   CAL_ERR_NONE            if successful
*           CAL_ERR_*               other errors.
*/
  cal_err_t cal_component_get_type (const cal_component_t *comp,
                                    cal_comp_type_t *type);


/********************************************************************
 * List properties, retrieve/add/remove properties                  *
 ********************************************************************/

/** @brief     list properties of the specified type.
*
* If the ptype is set to CAL_ANY_PROPERTY, all property ids will be returned.
* App should later free the array.
*
* @param[in]   comp         a pointer to the component.
* @param[in]   ptype        specify the property type
* @param[out]  pids         a pointer to the location where property ids will 
*                            be returned, the array is terminated by PID_INVALID_VALUE.
*
* @return   CAL_ERR_NONE            if successful
*           CAL_ERR_*               other errors.
*/  
    cal_err_t cal_component_get_properties (const cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          uint32_t **pids);
                                          

/** @brief     add a property of the specified type to the component 
*
* @param[in]   comp         a pointer to the component.
* @param[in]   ptype        specify which kind of property to add.   
* @param[out]  pid          newly added property's pid, not newly allocated, needn't
*														be freed
*
* @return   CAL_ERR_NONE            if successful
*           CAL_ERR_INVALID         if the ptype shouldn't appear in the component
*
*/
  cal_err_t cal_component_add_property (cal_component_t *comp, 
                                        cal_prop_type_t ptype,
                                        uint32_t *pid);

/** @brief     remove property from the component.
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property will be removed.
*
* @return   CAL_ERR_NONE            if successful
*           CAL_ERR_*               other errors.  
*/
  cal_err_t cal_component_remove_property (cal_component_t *comp,
                                           uint32_t pid);


/** @brief     remove properties of the specified type from the component.
*
* @param[in]   comp         a pointer to the component.
* @param[in]   ptype        specify which kind of property to remove.
*
* @return   CAL_ERR_NONE            if successful
*						CAL_ERR_NOT_FOUND				if there's no property of the type "ptype"
*           CAL_ERR_*               other errors.  
*/
  cal_err_t cal_component_remove_property_by_type (cal_component_t *comp,
                                           cal_prop_type_t ptype);         


/** @brief     get int32_t property value.
*
* Application shall get "pid" first by cal_component_get_properties() 
* or cal_component_add_property(), and then pass the "pid" into this function
* to get the property's value.
* 
*
* @param[in]   comp       a pointer to the component.
* @param[in]   pid          property id indicating which property's value to get, 
* @param[out]  value        a pointer to the location where the value of 
*                           the property will be returned.
*
* @return   CAL_ERR_NONE          if successful
*            CAL_ERR_NOT_FOUND    if the input "pid" can't be found in the component
*/
cal_err_t cal_property_get_int32_value (cal_component_t *comp,
                                          uint32_t pid,
                                          int32_t *value);                                          

/** @brief     set int32_t property value  .
* 
* Application shall get "pid" first by cal_component_get_properties() 
* or cal_component_add_property(), and then pass the "pid" into this function
* to set the property's value.
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's value to set.
* @param[in]   value        value to be set.
*
* @return   CAL_ERR_NONE          if successful
*   			  CAL_ERR_TOO_BIG			the specified value is too big
*           CAL_ERR_NOT_FOUND     if the input "pid" can't be found in the component 
*						CAL_ERR_INVALID       the specified value is invalid
*/
  cal_err_t cal_property_set_int32_value (cal_component_t *comp,
                                            uint32_t pid,  
                                            int32_t value);
                                            
/** @brief     get int64_t property value.
*
* Application shall get "pid" first by cal_component_get_properties() 
* or cal_component_add_property(), and then pass the "pid" into this function
* to get the property's value.
* 
*
* @param[in]   comp       a pointer to the component.
* @param[in]   pid          property id indicating which property's value to get, 
* @param[out]  value        a pointer to the location where the value of 
*                           the property will be returned.
*
* @return   CAL_ERR_NONE          if successful
*            CAL_ERR_NOT_FOUND    if the input "pid" can't be found in the component
*/
cal_err_t cal_property_get_int64_value (cal_component_t *comp,
                                          uint32_t pid,
                                          int64_t *value);                                          

/** @brief     set int64_t property value  .
* 
* Application shall get "pid" first by cal_component_get_properties() 
* or cal_component_add_property(), and then pass the "pid" into this function
* to set the property's value.
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's value to set.
* @param[in]   value        value to be set.
*
* @return   CAL_ERR_NONE          if successful
*   			  CAL_ERR_TOO_BIG			the specified value is too big
*           CAL_ERR_NOT_FOUND     if the input "pid" can't be found in the component 
*/
  cal_err_t cal_property_set_int64_value (cal_component_t *comp,
                                            uint32_t pid,  
                                            int64_t value);

/** @brief     get float (4 bytes) property value.
* 
* Please refer to "cal_component_get_integer_property_value()".
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's value to get, 
*                           the "first pid macro"s can also be passed in
* @param[out]  value        a pointer to the location where the value of 
*                           the property will be returned.
*
* @return   CAL_ERR_NONE          if successful
*           CAL_ERR_NOT_FOUND     if the input "pid" can't be found in the component
*/
  cal_err_t cal_property_get_float_value (cal_component_t *comp,
                                                    uint32_t pid,   
                                                    float *value);

                                          
/** @brief     set float (4 bytes) property value.
*
* Please refer to "cal_component_set_integer_property_value()".
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's value to set.
* @param[in]   value        value to be set.
*
* @return   CAL_ERR_NONE          if successful
*			      CAL_ERR_TOO_BIG			the specified value is too big
*           CAL_ERR_NOT_FOUND     if the input "pid" can't be found in the component
*/
  cal_err_t cal_property_set_float_value (cal_component_t *comp,
                                                    uint32_t pid,   
                                                    float value);

/** @brief     get double (8 bytes) property value.
* 
* Please refer to "cal_component_get_integer_property_value()".
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's value to get, 
*                           the "first pid macro"s can also be passed in
* @param[out]  value        a pointer to the location where the value of 
*                           the property will be returned.
*
* @return   CAL_ERR_NONE          if successful
*           CAL_ERR_NOT_FOUND     if the input "pid" can't be found in the component
*/
  cal_err_t cal_property_get_double_value (cal_component_t *comp,
                                                    uint32_t pid,   
                                                    double *value);

                                          
/** @brief     set double (8 bytes) property value.
*
* Please refer to "cal_component_set_integer_property_value()".
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's value to set.
* @param[in]   value        value to be set.
*
* @return   CAL_ERR_NONE          if successful
*			      CAL_ERR_TOO_BIG			the specified value is too big
*           CAL_ERR_NOT_FOUND     if the input "pid" can't be found in the component
*/
  cal_err_t cal_property_set_double_value (cal_component_t *comp,
                                                    uint32_t pid,   
                                                    double value);
/** @brief     get string property value .
*
* Please refer to "cal_component_get_integer_property_value()".
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's value to get, 
* @param[out]  value        a pointer to the location where the value of 
*                           the property will be returned.
*
* @return   CAL_ERR_NONE          if successful
*           CAL_ERR_NOT_FOUND     if the input "pid" can't be found in the component
* @note    The pointer to inner memory is returned directly. Application shouldn't free it.
*/
  cal_err_t cal_property_get_string_value (const cal_component_t *comp,
                                                     uint32_t pid,  
                                                     const char **value);

                                          
/** @brief     set string property value .
*
* Please refer to "cal_component_set_integer_property_value()".
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's value to set.
* @param[in]   value        value to be set.
*
* @return   CAL_ERR_NONE          if successful
*   			  CAL_ERR_TOO_BIG			the length of value is bigger than the max size specified 
*           CAL_ERR_NOT_FOUND     if the input "pid" can't be found in the component
*/
cal_err_t cal_property_set_string_value (cal_component_t *comp,
                                                   uint32_t pid,  
                                                   const char *value);


/** @brief     get property value whose type is binary.
*
* Please refer to "cal_component_get_integer_property_value()".
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's value to get, 
* @param[out]   size     		the length of the binary value
* @param[out]  value        a pointer to the location where the value of 
*                           the property will be returned.
*
* @return    CAL_ERR_NONE          if successful
*            CAL_ERR_NOT_FOUND     if the input "pid" can't be found in the component
* @note    The pointer to inner memory is returned directly. Application shouldn't free it.
*/
  cal_err_t cal_property_get_binary_value (const cal_component_t *comp,
                                                     uint32_t pid,
                                                     uint32_t *size,
                                                     const void **value);

                                          
/** @brief     set property value whose type is binary.
*
* Please refer to "cal_component_set_integer_property_value()".
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's value to set.
* @param[in]   size           the length of the binary value
* @param[in]   value        value to be set.
*
* @return    CAL_ERR_NONE          if successful
*			       CAL_ERR_TOO_BIG			the length of value is bigger than the max size specified 
*            CAL_ERR_NOT_FOUND     if the input "pid" can't be found in the component
*/
  cal_err_t cal_property_set_binary_value (cal_component_t *comp,
                                                     uint32_t pid,
                                                     uint32_t size,
                                                     const void *value);

/** @brief     get int32_t property value by specifying the property type .
*
* If the component has exactly one property of the property type, the property 
* value will be returned.
* If the component doesn't has property of the type, an error code CAL_ERR_NOT_FOUND 
* will be returned.
* If there's more than one property of the property type, 
* an error code CAL_ERR_INVALID will be returned.
*
* @param[in]   comp       a pointer to the component.
* @param[in]   ptype       property type 
* @param[out]  value        a pointer to the location where the value will be returned.
*
*@return     CAL_ERR_NONE          if successful
*						 CAL_ERR_INVALID			 the component has more than one property of the property type
*						 CAL_ERR_NOT_FOUND     the component doesn't has property of the type	
*						 CAL_ERR_NO_ALARM      if the input "ptype" is alarm component property type while 
*                                  				the component isn't an alarm components and it doesn't
*                                  				contain an alarm component
*            CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                  				the component contains multiple alarm component
*/
cal_err_t cal_proptype_get_int32_value (cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          int32_t *value); 
																					
/*@brief     set int32_t property value by specifying the property type
*
* If the component has exactly one property of the property type, the property 
* value will be set.
* If the component doesn't has property of the type, a property of the type will 
* be automatically added to the component first. 
* If there's more than one property of the property type, 
* an error code CAL_ERR_INVALID will be returned.
*
* @param[in]   comp        a pointer to the component.
* @param[in]   ptype       property type 
* @param[in]   value       value to be set.
*
* @return   CAL_ERR_NONE          if successful
*           CAL_ERR_INVALID			 there's more than one property of the property type
*   			  CAL_ERR_TOO_BIG			 the specified value is too big
*           CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                  the component contains multiple alarm component
*/
  cal_err_t cal_proptype_set_int32_value (cal_component_t *comp,
                                            cal_prop_type_t ptype,  
                                            int32_t value);   
/** @brief     get int64_t property value by specifying the property type .
*
* If the component has exactly one property of the property type, the property 
* value will be returned.
* If the component doesn't has property of the type, an error code CAL_ERR_NOT_FOUND 
* will be returned.
* If there's more than one property of the property type, 
* an error code CAL_ERR_INVALID will be returned.
*
* @param[in]   comp       a pointer to the component.
* @param[in]   ptype       property type 
* @param[out]  value        a pointer to the location where the value will be returned.
*
*@return     CAL_ERR_NONE          if successful
*						 CAL_ERR_INVALID			 the component has more than one property of the property type
*						 CAL_ERR_NOT_FOUND     the component doesn't has property of the type	
*						 CAL_ERR_NO_ALARM      if the input "ptype" is alarm component property type while 
*                                  				the component isn't an alarm components and it doesn't
*                                  				contain an alarm component
*            CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                  				the component contains multiple alarm component
*/
cal_err_t cal_proptype_get_int64_value (cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          int64_t *value); 
/*@brief     set int64_t property value by specifying the property type
*
* If the component has exactly one property of the property type, the property 
* value will be set.
* If the component doesn't has property of the type, a property of the type will 
* be automatically added to the component first. 
* If there's more than one property of the property type, 
* an error code CAL_ERR_INVALID will be returned.
*
* @param[in]   comp         a pointer to the component.
* @param[in]   ptype       property type 
* @param[in]   value        value to be set.
*
* @return   CAL_ERR_NONE          if successful
*           CAL_ERR_INVALID			 there's more than one property of the property type
*   			  CAL_ERR_TOO_BIG			 the specified value is too big
*           CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                  the component contains multiple alarm component
*/
  cal_err_t cal_proptype_set_int64_value (cal_component_t *comp,
                                            cal_prop_type_t ptype,  
                                            int64_t value); 																						

/** @brief     get float(4 bytes) property value by specifying the property type .
*
* Please refer to "cal_component_get_integer_property_value_by_type()".
*
* @param[in]   comp       a pointer to the component.
* @param[in]   ptype       property type 
* @param[out]  value        a pointer to the location where the value will be returned.
*
*@return     CAL_ERR_NONE          if successful
*						 CAL_ERR_INVALID			 the component has more than one property of the property type
*						 CAL_ERR_NOT_FOUND     the component doesn't has property of the type	
*						 CAL_ERR_NO_ALARM      if the input "ptype" is alarm component property type while 
*                                  				the component isn't an alarm components and it doesn't
*                                  				contain an alarm component
*            CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                  				the component contains multiple alarm component
*/
	cal_err_t cal_proptype_get_float_value (cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          float *value);                                                                                                                                       


/** @brief     set float(4 bytes) property value by specifying the property type .
*
* Please refer to "cal_component_set_integer_property_value_by_type()".
*
* @param[in]   comp       a pointer to the component.
* @param[in]   ptype       property type 
* @param[out]  value       value to be set.
*
* @return   CAL_ERR_NONE          if successful
*           CAL_ERR_INVALID			 there's more than one property of the property type
*   			  CAL_ERR_TOO_BIG			 the specified value is too big
*           CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                  the component contains multiple alarm component
*/
	cal_err_t cal_proptype_set_float_value (cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          float *value);	
																					
/** @brief     get double(8 bytes) property value by specifying the property type .
*
* Please refer to "cal_component_get_integer_property_value_by_type()".
*
* @param[in]   comp       a pointer to the component.
* @param[in]   ptype       property type 
* @param[out]  value        a pointer to the location where the value will be returned.
*
*@return     CAL_ERR_NONE          if successful
*						 CAL_ERR_INVALID			 the component has more than one property of the property type
*						 CAL_ERR_NOT_FOUND     the component doesn't has property of the type	
*						 CAL_ERR_NO_ALARM      if the input "ptype" is alarm component property type while 
*                                  				the component isn't an alarm components and it doesn't
*                                  				contain an alarm component
*            CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                  				the component contains multiple alarm component
*/
	cal_err_t cal_proptype_get_double_value (cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          double *value);                                                                                                                                       


/** @brief     set double(8 bytes) property value by specifying the property type .
*
* Please refer to "cal_component_set_integer_property_value_by_type()".
*
* @param[in]   comp       a pointer to the component.
* @param[in]   ptype       property type 
* @param[out]  value       value to be set.
*
* @return   CAL_ERR_NONE          if successful
*           CAL_ERR_INVALID			 there's more than one property of the property type
*   			  CAL_ERR_TOO_BIG			 the specified value is too big
*           CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                  the component contains multiple alarm component
*/
	cal_err_t cal_proptype_set_double_value (cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          double *value);																						
                                          
/** @brief     get string property value by specifying the property type .
*
* Please refer to "cal_component_get_integer_property_value_by_type()".
*
* @param[in]   comp       a pointer to the component.
* @param[in]   ptype       property type 
* @param[out]  value        a pointer to the location where the value will be returned.
*
*@return     CAL_ERR_NONE          if successful
*						 CAL_ERR_INVALID			 the component has more than one property of the property type
*						 CAL_ERR_NOT_FOUND     the component doesn't has property of the type	
*						 CAL_ERR_NO_ALARM      if the input "ptype" is alarm component property type while 
*                                  				the component isn't an alarm components and it doesn't
*                                  				contain an alarm component
*            CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                  				the component contains multiple alarm component
*/
	cal_err_t cal_proptype_get_string_value (cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          const char **value);  
                                          
/** @brief     set string property value by specifying the property type .
*
* Please refer to "cal_component_set_integer_property_value_by_type()".
*
* @param[in]   comp       a pointer to the component.
* @param[in]   ptype       property type 
* @param[out]  value       value to be set.
*
* @return   CAL_ERR_NONE          if successful
*           CAL_ERR_INVALID			 there's more than one property of the property type
*   			  CAL_ERR_TOO_BIG			 the specified value is too big
*           CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                 the component contains multiple alarm component
*/
	cal_err_t cal_proptype_set_string_value (cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          const char *value);
                                          
/** @brief     get binary property value by specifying the property type .
*
* Please refer to "cal_component_get_integer_property_value_by_type()".
*
* @param[in]   comp       a pointer to the component.
* @param[in]   ptype       property type 
* @param[out]   size        the length of the binary value
* @param[out]  value        a pointer to the location where the value will be returned.
*
*@return     CAL_ERR_NONE          if successful
*						 CAL_ERR_INVALID			 the component has more than one property of the property type
*						 CAL_ERR_NOT_FOUND     the component doesn't has property of the type	
*						 CAL_ERR_NO_ALARM      if the input "ptype" is alarm component property type while 
*                                  				the component isn't an alarm components and it doesn't
*                                  				contain an alarm component
*            CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                  				the component contains multiple alarm component
*/
	cal_err_t cal_proptype_get_binary_value (cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          uint32_t *size,
                                          const void **value);                                          
                                                                                                                            																									 
/** @brief     set binary property value by specifying the property type .
*
* Please refer to "cal_component_set_integer_property_value_by_type()".
*
* @param[in]   comp         a pointer to the component.
* @param[in]   ptype       property type 
* @param[in]   size          the length of the binary value
* @param[in]   value        value to be set.
*
* @return   CAL_ERR_NONE          if successful
*           CAL_ERR_INVALID			 there's more than one property of the property type
*   			  CAL_ERR_TOO_BIG			 the specified value is too big
*           CAL_ERR_MULTI_ALARMS  if the input "ptype" is alarm component property type while 
*                                 the component contains multiple alarm component
*/
cal_err_t cal_proptype_set_binary_value					 (cal_component_t *comp,
                                                   cal_prop_type_t ptype,  
																									 uint32_t size,
                                                   const void *value);	

/********************************************************************
 * List property value type and label                                *
 ********************************************************************/
/** @brief     get the property type and value type for a given property id
*              in the specified component.
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          specify the property.         
* @param[out]  ptype        return the property type of the property specified by pid. 
* @param[out]  vtype        return the property value type of the property specified by pid.
*
* @return   CAL_ERR_NONE              if successful
*/
  cal_err_t cal_property_get_type           (const cal_component_t *comp,
                                             uint32_t pid,  
                                             cal_prop_type_t *ptype,
                                             cal_prop_value_type_t *vtype);
                                             
 
/** @brief     get the property value type for a property type.
*
* For each type, LIPS specification specify the value type
*
* @param[in]   ptype        the property type. 
* @param[out]  vtype        return the property value type of the property type.
*
* @return   CAL_ERR_NONE          if successful
*/
  cal_err_t cal_property_get_valuetype (cal_prop_type_t ptype,
                                        cal_prop_value_type_t *vtype);

/** @brief   get the property lable for a property type.
*
* For each type, LIPS specification specify the default label
*
* @param[in]   ptype        the property type. 
* @param[out]  label        return the pointer to the location where the label is returned.
*
* @return   CAL_ERR_NONE          if successful
*/
  cal_err_t cal_property_get_label       (cal_prop_type_t ptype,
                                         const char **label);

/** @brief   get the maximum size of the value for a property type.
*
* For each property type, the enabler specify the maximum size of the value
*
* @param[in]   ptype        the property type. 
* @param[out] size        return the pointer to the location where the size is returned.
*
* @return   CAL_ERR_NONE          if successful
*/
  cal_err_t cal_property_get_max_size       (cal_prop_type_t ptype,
                                             uint32_t	*size);

    
/********************************************************************
 * Create or remove extension property type  								*
 ********************************************************************/
 /** @brief   list the property types in a backend.
 *
 * This function will list not only the standard  but also the extension property types
 *
 * @param[in]  sid          session id.
 * @param[out] ptypes    return an array of property type
 *
 * @return   CAL_ERR_NONE          if  successful;
 */
   cal_err_t cal_list_property_types (cal_sid_t sid,  cal_prop_type_t **ptypes);

/** @brief   create an extension property type
*
* Except for standard property type, application can also create extension 
* property types. By inputing the "label" and the "value type" , a newly created "ptype" will 
* be allocated in this function. The the property type can be used the same as standard property
* type.
*
* @param[in]  sid         session id.
* @param[in]  label      label of the property type.
* @param[in]  vtype     value type of the property type. 
* @param[in]  size       maximum size of the value for the property type. 
* @param[out] ptype    return a newly-allocated property type
*
* @return   CAL_ERR_NONE          if successful;
*/
   cal_err_t cal_create_property_type (cal_sid_t sid,  
   									     const char *label,
   									     cal_prop_value_type_t vtype,
   									     uint32_t	size,
   									     cal_prop_type_t *ptype);

/** @brief   remove an extension property type
*
* Extension property type can be removed by this function.
*
* @param[in]  sid         session id.
* @param[in] ptype      the property type to remove
*
* @return   CAL_ERR_NONE          if successful;
*/
   cal_err_t  cal_remove_property_type (cal_sid_t sid,
   										 cal_prop_type_t ptype);
                                              
/****************************************
 * Count/get/set parameter              *
 ****************************************/
/* Besides its own value, a property may have several parameters. 
 * The type of the parameters can be the value of enum type "cal_parameter_t".
 * For each type, LIPS specification gives a default label which can be got by:
 *  cal_parameter_get_label ();
 *
 * 
 * Some of the parameters may have multiple values. As specified in RFC2445,
 * there're 3 parameters which may contain multiple values:
 * delegators, delegatees, group/list membership.
 * For example:
 * ATTENDEE;DELEGATED-TO="MAILTO:jdoe@host.com","MAILTO:jqpublic@host.com":MAILTO:jsmith@host.com
 * Multiple values will be concatenated wich ",", and be get/set as a string.
 * 
 */
 
 /** @brief     get parameters for the specified property.
 *
 * @param[in]     comp       a pointer to the component.
 * @param[in]     pid        property id specify the property.
 * @param[out]    patypes    return an array of the parameter types in the property.
 *                           The array is terminated by CAL_INVALID_PARAMETER.
 *
 * @return    CAL_ERR_NONE          if successful
 *            CAL_ERR_NOT_FOUND     if the input "pid" can't be found in the component
 */
  cal_err_t cal_property_get_parameters (const cal_component_t *comp,
                                         uint32_t pid,
                                         cal_parameter_type_t **patypes);    
	

/** @brief   get the parameter label for a parameter type
*
* @param[in]   patype       the parameter type. 
* @param[out]  label        return the pointer to the location where the label is returned.
*
* @return   CAL_ERR_NONE         if successful

*/
  cal_err_t cal_parameter_get_label (cal_parameter_type_t patype,
                                     char **label);

/** @brief     get the maximum size of the value for a parameter type.
*
* @param[in]     patype     specify the parameter type.
* @param[out]    size      return the pointer to the location where the maximum size is returned
*
* @return   CAL_ERR_NONE          if successful
*/
  cal_err_t cal_parameter_get_max_size     (cal_parameter_type_t patype,
                                         uint32_t	*size);

/** @brief     get value from the specified property.
*
* @param[in]     comp       a pointer to the component.
* @param[in]     pid        property id indicating which property's parameter to get, 
*                           the "first pid macro"s can also be passed in
* @param[in]     patype     specify the parameter type.
* @param[out]    value      return value; if there're multi value for the parameter, 
*                           the values will be concatenated by ",".
*
* @return   CAL_ERR_NONE          if successful
* 				  CAL_ERR_NOT_FOUND     the component doesn't has property specified by "pid",
*																	or the property doesn't has parameter of the type "patype"
*/
  cal_err_t cal_parameter_get_value     (const cal_component_t *comp, 
                                         uint32_t pid,
                                         cal_parameter_type_t patype,
                                         const char **value);


                                                        
/** @brief     set parameter of the specified property.
*
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's parameter to set.
* @param[in]   patype       specify the parameter type.
* @param[in]   value        values to set. If it's set to NULL,the former value will be unset.
*
* @return    CAL_ERR_NONE          if successful
*            CAL_ERR_NOT_FOUND     if the component doesn't has property specified by "pid"
*
* @note    Multi values should be concatenated to one string by ",". Then the 
*          string can be passed to the function.
*/
  cal_err_t cal_parameter_set_value       (const cal_component_t *comp,                                                       
                                           uint32_t pid,
                                           cal_parameter_type_t patype,
                                           const char *value);

	/** @brief     get parameters by specifying property type
 *
 * @param[in]     comp       a pointer to the component.
 * @param[in]   	ptype       property type 
 * @param[out]    patypes    return an array of the parameter types in the property.
 *                           The array is terminated by CAL_INVALID_PARAMETER.
 *
 * @return    CAL_ERR_NONE          if successful
 *						CAL_ERR_INVALID       the component has more than one property of the type "ptype"
 *	 				  CAL_ERR_NOT_FOUND     the component doesn't has property of the type	"ptype"
 */
  cal_err_t cal_proptype_get_parameters (const cal_component_t *comp,
                                         cal_prop_type_t ptype,
                                         cal_parameter_type_t **patypes);    


/** @brief     get parameter value by specifying the property type.
*
* If the component has only one propery of the "ptype", the value will be got from 
*	the property's parameter of the type "patype"; If the component doesn't has property
* of the "ptype", CAL_ERR_NOT_FOUND will be returned; If the component has more than 
* one property of the property type, CAL_ERR_INVALID will be returned.
*
* @param[in]     comp       a pointer to the component.
* @param[in]   	 ptype       property type 
* @param[in]     patype     specify the parameter type.
* @param[out]    value      return value; if there're multi value for the parameter, 
*                           the values will be concatenated by ",".
*
* @return   CAL_ERR_NONE          if successful
*						CAL_ERR_INVALID       the component has more than one property of the type "ptype"
*	 				  CAL_ERR_NOT_FOUND     the component doesn't has property of the type "ptype"
*						CAL_ERR_INVALID			  the component has more than one property of the property type
*/
   cal_err_t cal_proptype_parameter_get_value     (const cal_component_t *comp, 
                                         					cal_prop_type_t ptype,
                                         					cal_parameter_type_t patype,
                                         					const char **value);
																				 
/** @brief     set parameter value by specifying the property type.
*
* If the component has only one propery of the "ptype", the value will be set for 
*	the property's parameter of the type "patype"; If the component doesn't has property
* of the "ptype", CAL_ERR_NOT_FOUND will be returned; If the component has more than 
* one property of the property type, CAL_ERR_INVALID will be returned.
*
* @param[in]   comp         a pointer to the component.
* @param[in]   ptype        property type 
* @param[in]   patype       specify the parameter type.
* @param[in]   value        values to set. If it's set to NULL,the former value will be unset.
*
* @return    CAL_ERR_NONE          if successful
*						CAL_ERR_INVALID       the component has more than one property of the type "ptype"
*	 				  CAL_ERR_NOT_FOUND     the component doesn't has property of the type "ptype"
*           CAL_ERR_INVALID			 	 the component has more than one property of the property type
*            
* @note    Multiple values should be concatenated to one string by ",". Then the 
*          string can be passed to the function.
*/
  cal_err_t cal_proptype_parameter_set_value       (const cal_component_t *comp,                                                       
                                           					cal_prop_type_t ptype,
                                           					cal_parameter_type_t patype,
                                           					const char *value);																				 
/********************************************************************
 * Create or remove extension parameter type  											*
 ********************************************************************/
 /** @brief   list the parameter types in a backend.
 *
 * This function will list not only the standard  but also the extension parameter types
 *
 * @param[in]  sid          session id.
 * @param[out] patypes    return an array of parameter type
 *
 * @return   CAL_ERR_NONE          if  successful;
 */
   cal_err_t cal_list_parameter_types (cal_sid_t sid,  cal_parameter_type_t **patypes);

/** @brief   create an extension parameter type
*
* Except for standard parameter type, application can also create extension 
* parameter types. By inputing the "label", a newly created "patype" will 
* be allocated in this function. Then the property type can be used the same as standard parameter
* type.
*
* @param[in]  sid         session id.
* @param[in]  label      label of the parameter type.
* @param[in]  size		   the maximum size of the value for the parameter type
* @param[out] patype    return a newly-allocated parameter type
*
* @return   CAL_ERR_NONE          if successful;
*/
   cal_err_t cal_create_parameter_type (cal_sid_t sid,  
   									     const char *label,
   									     uint32_t	size,
   									     cal_parameter_type_t *patype);

/** @brief   remove an extension parameter type
*
* Extension parameter type can be removed by this function.
*
* @param[in]  sid         session id.
* @param[in]  patype      the parameter type to remove
*
* @return   CAL_ERR_NONE          if successful;
*/
   cal_err_t  cal_remove_parameter_type (cal_sid_t sid,
   										 cal_parameter_type_t patype);
                                              
/********************************************************************
 * Conversion functions                                              *
 ********************************************************************/
/** @brief     create a new component from an iCalendar string.
*
* @param[in]  	sid         session id.
* @param[in]     string    a string conform to RFC2445 representing an iCalendar
*                          component.
* @param[out]    comp      a pointer to the memory location where a newly created 
*                          component will be returned. 
*
* @return   CAL_ERR_NONE          if successful
*
* @note     applications should later free the component using the 
*           cal_component_free() call.
*/
  cal_err_t cal_component_new_from_string (cal_sid_t sid, const char *string,
                                           cal_component_t **comp);

/** @brief     convert the component to a iCalendar string conform to RFC2445.
*
* @param[in]  	sid         	  session id.
* @param[in]     comp        a pointer to the component.
* @param[out]    string      a string representation of an iCalendar component.
*
* @return   CAL_ERR_NONE          if successful
*
* @note     applications should later free the string when no longer needed.
*/
  cal_err_t cal_component_to_string (cal_sid_t sid, const cal_component_t *comp,
                                     char **string);
                                     
/***************************************************
 * Retrieve alarms, add/remove alarm               *
 ***************************************************/
/** @brief    get the alarm components in a component.
* 
* The component type can only be CAL_EVENT_COMPONENT or CAL_TODO_COMPONENT.
*
* @param[in]    comp          a pointer to the component.
* @param[out]   alarms        pointer to the alarm components array returned. The returned 
*                             array is terminated by NULL.
*
* @return    CAL_ERR_NONE              if  successful;
*            CAL_ERR_INVALID           if the "comp" type isn't CAL_EVENT_COMPONENT 
*                                     or CAL_TODO_COMPONENT .
*
* @note     In this function, the pointer to the inner component will be 
*           returned directly. The memory is maintained by the enabler. 
*           Application shouldn't free the memory. But application can modify 
*           the subcompoent content.
*/
  cal_err_t cal_component_get_alarms (const cal_component_t *comp,
                                      cal_component_t ***alarms);
   
/*added by yl 20071025
	The alarm component is newly created in this function. The caller should free
	the memory when it's not needed any more.
	CAL_ERR_NOT_FOUND    if there's not alarm subcompoent
*/	
	cal_err_t cal_component_get_alarm (const cal_component_t *comp,
                                      cal_component_t **alarm);
/** @brief     Add an alarm  into the component.
*
*
* @param[in]    comp         pointer to the component.
* @param[in]    alarm        pointer to the alarm component.
*
* @return    CAL_ERR_NONE              if successful;
*            CAL_ERR_INVALID           if the "comp" type isn't CAL_EVENT_COMPONENT 
*                                     or CAL_TODO_COMPONENT
*/
  cal_err_t cal_component_add_alarm  (cal_component_t *comp,
                                      const cal_component_t *alarm);

/** @brief     Remove an alarm from a component.
*
* @param[in]    comp         pointer to the component.
* @param[in]    alarm        pointer to the alarm component.
*
* @return    CAL_ERR_NONE              if successful;
*            CAL_ERR_INVALID           if the "comp" type isn't CAL_EVENT_COMPONENT 
*                                     or CAL_TODO_COMPONENT 
*/
/*  cal_err_t cal_component_remove_alarm (cal_component_t *comp,
                                         const cal_component_t *alarm);*/


/*added by yl 20071025*/
	cal_err_t cal_component_remove_alarm (cal_component_t *comp);
                                                                   
#ifdef __cplusplus
}
#endif                          /* __cplusplus */
#endif                          /* _LIPS_CAL_COMPONENT_H */

/*
vi:ts=2:nowrap:ai:expandtab
*/
