/**  
 *  @file <caltype.h> 
 *  @brief some basic types for libcalenabler2
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *    notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */
#ifndef  __LIPS_CAL_TYPE_H
#define  __LIPS_CAL_TYPE_H

#include <stdint.h>
#include <glib.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define PID_INVALID_VALUE 0


/** @brief boolean type. */  
typedef   gboolean    cal_boolean_t;

  
/** @brief a unique identifier of uint32_t type, used to specify a session. */  
typedef  uint32_t        cal_sid_t;


/**@brief  property type*/
typedef uint32_t 		cal_prop_type_t;


/**@brief  parameter type*/
typedef uint32_t 		cal_parameter_type_t;



typedef char * cal_channel_t;

/** 
 * @brief  action property type
 */
typedef enum 
{
  CAL_ACTION_X = 10000,
	CAL_ACTION_AUDIO = 10001,
  CAL_ACTION_DISPLAY = 10002,
  CAL_ACTION_EMAIL = 10003,
  CAL_ACTION_PROCEDURE = 10004,
  CAL_ACTION_NONE = 10005
}cal_action_property_t;

/** 
 * @brief  class property type
 */
typedef enum {
  CAL_CLASS_X = 10011,
  CAL_CLASS_PUBLIC = 10012,
  CAL_CLASS_PRIVATE = 10013,
  CAL_CLASS_CONFIDENTIAL = 10014,
  CAL_CLASS_NONE = 10015
}cal_class_property_t;

/** 
 * @brief  status property type
 */
typedef enum {
  CAL_STATUS_X = 10051,
  CAL_STATUS_TENTATIVE = 10052,
  CAL_STATUS_CONFIRMED = 10053,
  CAL_STATUS_COMPLETED = 10054,
  CAL_STATUS_NEEDSACTION = 10055,
  CAL_STATUS_CANCELLED = 10056,
  CAL_STATUS_INPROCESS = 10057,
  CAL_STATUS_DRAFT = 10058,
  CAL_STATUS_FINAL = 10059,
  CAL_STATUS_NONE = 10060
} cal_status_property_t;

/** 
 * @brief  transparent property type
 */
typedef enum {
  CAL_TRANSP_X = 10061,
  CAL_TRANSP_OPAQUE = 10062,
  CAL_TRANSP_OPAQUENOCONFLICT = 10063,
  CAL_TRANSP_TRANSPARENT = 10064,
  CAL_TRANSP_TRANSPARENTNOCONFLICT = 10065,
  CAL_TRANSP_NONE = 10066
} cal_transp_property_t;
  
/** 
 * @brief  Refer to RFC2445 4.2.12
 */
  typedef enum
  {
    CAL_PARTSTAT_NEEDS_ACTION = 10070,
    CAL_PARTSTAT_ACCEPTED = 10071,
    CAL_PARTSTAT_DECLINED = 10072,
    CAL_PARTSTAT_TENTATIVE = 10073,
    CAL_PARTSTAT_DELEGATED = 10074,      
    CAL_PARTSTAT_COMPLETED = 10075,
    CAL_PARTSTAT_INPROCESS = 10076
  } cal_partstat_t;

    
/**
 * @brief  Refer to RFC2446 3
 */
  typedef enum
  {
    CAL_METHOD_PUBLISH = 10031,
    CAL_METHOD_REQUEST = 10032,
    CAL_METHOD_REPLY = 10033,
    CAL_METHOD_ADD = 10034,      
    CAL_METHOD_CANCEL = 10035,   
    CAL_METHOD_REFRESH = 10036,  
    CAL_METHOD_COUNTER = 10037,  
    CAL_METHOD_DECLINECOUNTER  = 10038
  } cal_method_type_t;
  
  
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __LIPS_CAL_TYPE_H */

/*
vi:ts=2:nowrap:ai:expandtab
*/
