/*  libcalenabler2 - Library of LiPS address book enabler
 *
 *Authors: YANG Lin <lin.yang@orange-ftgroup.com>
 *
 *This software and associated documentation files (the "Software") 
 *are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *All Rights Reserved. 
 *
 *A copyright license is hereby granted for redistribution and use of 
 *the Software in source and binary forms, with or without modification, 
 *provided that the following conditions are met: 
 *- Redistributions of source code must retain the above copyright notice, 
 *this copyright license and the following disclaimer. 
 *- Redistributions in binary form must reproduce the above copyright 
 *notice, this copyright license and the following disclaimer in the 
 *documentation and/or other materials provided with the distribution. 
 *- Neither the name of LiPS  nor the names of its Members may be used 
 *to endorse or promote products derived from the Software without 
 *specific prior written permission. 
 *
 *A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *distribute any implementation compliant with the any specification adopted 
 *by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *and non-discriminatory terms and conditions and under reciprocity, as 
 *regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *POSSIBILITY OF SUCH DAMAGE. 
 */


#ifndef	_CAL_PRIVATE_H
#define _CAL_PRIVATE_H

#include <glib.h>

#include <caltype.h>
#include <cal.h>

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */



typedef enum cal_storage {
  CAL_DB    = 0,  /**<  Use database to store */
  CAL_FILE  ,  /**<  use file to store */
  //CAL_NETWROK    /**<  Network Address Book */
  } cal_storage_t;

  typedef struct _CalBackend CalBackend;
  typedef struct _CalendarFuncs CalendarFuncs;


//  extern evt_src_t *cal_evt_src;

  struct _CalBackend
  {
    gint32 id;
    cal_type_t type;  /**<CAL_TYPE_EVENT, CAL_TYPE_TODO, CAL_TYPE_ALARM, CAL_TYPE_JOURNAL   */
		guint32 stype; /*CAL_DB: database; CAL_FILE: file*/
    gchar *source;
    gchar *name;
    gchar *prodid;
    gchar *version;
    gchar *calscale;
    gchar *defaulttz;  /**<default timezone */
    gint ref;
    CalendarFuncs *funcs;
    gpointer priv;
  };

  struct _CalendarFuncs
  {
    cal_err_t (*cal_initialize) (CalBackend *bk);
    cal_err_t (*cal_finalize) (CalBackend *bk);

    
    cal_err_t (*cal_add_component) (CalBackend *bk, 
																		cal_component_t *component, char **uid);
                                    
		cal_err_t (*cal_get_components_by_time) (CalBackend *bk, time_t start,
                                        time_t end, gboolean detached,
                                        cal_component_t ***components);
                                        
		cal_err_t (*cal_get_component_by_id) (CalBackend *bk,const char *uid,
                                      	  const char *rid,cal_component_t **component);
                                          
    cal_err_t (*cal_remove_component) (CalBackend *bk, const char *uid);
    
    cal_err_t (*cal_update_component) (CalBackend *bk,
                                   		 const cal_component_t *component); 
                                       
    cal_err_t (*cal_update_instance) (CalBackend *bk,
                                   		const cal_component_t *component,
                                      cal_mod_type_t modtype);
                                      
    cal_err_t (*cal_remove_instance) (CalBackend *bk, const char *uid, const char *rid,
                                      cal_mod_type_t modtype); 
																	
		cal_err_t (*cal_instance_is_multiple) (CalBackend *bk, const cal_component_t *instance,
                                    cal_boolean_t *bool);
  };

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* _CAL_PRIVATE_H */

/* 
vi:ts=2:nowrap:ai:expandtab
*/
