/*  libcalenabler2 - Library of LiPS calendar enabler
 *
 *Authors: Yang Lin <lin.yang@orange-ftgroup.com>
 *
 *This software and associated documentation files (the "Software") 
 *are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *All Rights Reserved. 
 *
 *A copyright license is hereby granted for redistribution and use of 
 *the Software in source and binary forms, with or without modification, 
 *provided that the following conditions are met: 
 *- Redistributions of source code must retain the above copyright notice, 
 *this copyright license and the following disclaimer. 
 *- Redistributions in binary form must reproduce the above copyright 
 *notice, this copyright license and the following disclaimer in the 
 *documentation and/or other materials provided with the distribution. 
 *- Neither the name of LiPS  nor the names of its Members may be used 
 *to endorse or promote products derived from the Software without 
 *specific prior written permission. 
 *
 *A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *distribute any implementation compliant with the any specification adopted 
 *by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *and non-discriminatory terms and conditions and under reciprocity, as 
 *regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <sqlite3.h>

#include <config.h>
#include <cal.h>
#include <calerror.h>

#include "cal-priv.h"
#include "cal-db.h"
#include "calenv.h"

#define	CALENDAR_ADMIN_DB_DISCOVERY_SQL		"select * from calendar;"
#define CALENDAR_INFO_DB_DISCOVERY_SQL    "select * from calendar;"
#define SESSION_RANGE		100

static GPtrArray *backends = NULL; //Stores backends opened
static GHashTable *session_maps = NULL; // The key: session id; the value: index
guint32 session_count = 0;


static CalBackend *
cal_bk_new (void)
{
  CalBackend *bk = NULL;

  bk = g_new0 (CalBackend, 1);
  if (bk == NULL)
  {
    return NULL;
  }

  return bk;
}

static void
cal_bk_free (CalBackend *bk)
{
  if (bk == NULL)
  {
    return;
  }

  if (bk->source)
  {
    g_free (bk->source);
  }
	if (bk->name)
  {
    g_free (bk->name);
  }
	if (bk->prodid)
  {
    g_free (bk->prodid);
  }
	if (bk->version)
  {
    g_free (bk->version);
  }
  if (bk->calscale)
  {
    g_free (bk->calscale);
  }
	if (bk->defaulttz)
  {
    g_free (bk->defaulttz);
  }
  if (bk->funcs)
  {
    g_free (bk->funcs);
  }

  g_free (bk);

  return;
}

void
cal_bk_print (const CalBackend *bk)
{
  g_print ("CalBackend[0x%08x]\n", (guint32) bk);
  g_print ("|ID     : %d\n", bk->id);
  g_print ("|TYPE   : %d\n", bk->type);
  g_print ("|SOURCE : %s\n", bk->source);
  g_print ("|REF    : %d\n", bk->ref);
  g_print ("`FUNCS  : 0x%08x\n", (guint32) bk->funcs);

 /* if (bk->funcs)
  {
    g_print (" |cal_initialize                 : 0x%08x\n",
             (guint32) bk->funcs->cal_initialize);
    g_print (" |cal_finalize                   : 0x%08x\n",
             (guint32) bk->funcs->cal_finalize);
    
  }*/

  return;
}

static int
_cal_info_discover_cb (void *unused, int argc, char **argv, char **azColName)
{
  if (argc == 5)
    {
     CalBackend *bk = (CalBackend *) unused;
     
     bk->name = g_strdup (argv[0]);
     bk->prodid = g_strdup (argv[1]);
     bk->version = g_strdup (argv[2]);
     bk->calscale = g_strdup (argv[3]);
     bk->defaulttz = g_strdup (argv[4]);
#ifdef DEBUG
    g_print ("%s() : bk->name = %s\n", __FUNCTION__, bk->name);
#endif
    }
   return 0;
}

/* Used as a callback for sqlite3 to add backends to the backends array */
static int
_cal_discover_cb (void *unused, int argc, char **argv, char **azColName)
{
  if (argc == 4)
  {
    CalBackend *bk = cal_bk_new ();

    bk->id = atoi (argv[0]);
    bk->type = atoi (argv[1]);
    bk->stype = atoi (argv[2]);
    bk->source = g_strdup (argv[3]);
    bk->ref = 0;
    bk->funcs = NULL;
#ifdef DEBUG
    g_print ("%s(): bk->stype = %d\n", __FUNCTION__, bk->stype);
#endif
    switch (bk->stype)
      {
      case CAL_DB : 
			{
				int ret;
        gchar *db_path = NULL;
        sqlite3 *database = NULL;
        gchar *err_str = NULL;
        

        db_path = g_build_filename (_env_get_source_prefix_db (), bk->source, NULL);
        if (sqlite3_open (db_path, &database))
          {
          g_print ("%s(): open admin database failed. %s\n", __FUNCTION__,
                  sqlite3_errmsg (database));

          sqlite3_close (database);
          return CAL_ERR_DB_TRANS;
          }
        ret =
           sqlite3_exec (database, CALENDAR_INFO_DB_DISCOVERY_SQL, _cal_info_discover_cb,
                             bk, &err_str);
                             
        if (ret != SQLITE_OK)
          {
          g_print ("%s(): discovery failed. [%s]\n", __FUNCTION__, err_str);
          g_free (err_str);
          return CAL_ERR_DB_TRANS;
          }
				g_free (db_path);
         sqlite3_close (database);
        }            
      break;
      case 2:
      break;
      }

    g_ptr_array_add (backends, (gpointer) bk);
  }
	
	
  return 0;
}


/* Discovers the backends from a given admin database*/
static cal_err_t
_cal_discover_bk (const gchar *adim_fname)
{
  sqlite3 *database = NULL;
  gchar *err_str = NULL;
  int ret;

#ifdef DEBUG
  g_print ("%s(): admin_fname = %s\n", __FUNCTION__, adim_fname);
#endif
  if (!g_file_test (adim_fname, G_FILE_TEST_IS_REGULAR))  // Test: database file exists
  {
    g_print ("%s(): admin database lost.\n", __FUNCTION__);
    return CAL_ERR_INVALID;
  }

  if (sqlite3_open (adim_fname, &database))  // Try to open database
  {
    g_print ("%s(): open admin database failed. %s\n", __FUNCTION__,
             sqlite3_errmsg (database));

    sqlite3_close (database);

    return CAL_ERR_DB_TRANS;
  }
	
/* Database successfully opened;
	 Call a SQL procedure to get all the backend records from the admin database.
	 Then in _cal_discover_cb (), parse the records into structures and insert the 
	 structures into backends array.
*/
  ret =
    sqlite3_exec (database, CALENDAR_ADMIN_DB_DISCOVERY_SQL, _cal_discover_cb,
                  NULL, &err_str);
  if (ret != SQLITE_OK)
  {
    g_print ("%s(): discovery failed. [%s]\n", __FUNCTION__, err_str);
    g_free (err_str);
    return CAL_ERR_DB_TRANS;
  }

  sqlite3_close (database);

  return CAL_ERR_NONE;
}

//load all the backends bk into the "backends"
cal_err_t _cal_load_backends_bk (void)
{
	const gchar *db_fname = NULL;
	cal_err_t ret;
	
	db_fname = _env_get_admin_database (); // Retrieve DB name from the environment
	ret = _cal_discover_bk (db_fname);  // Find the backends in this database
	
	return ret;
}

/*load all the backend bk into "backends"; initialize the "session_maps"*/
cal_err_t
_cal_initialize (void)
{
  
  cal_err_t ret;

	
  if (!(_env_initialize ()))  // Try to initialize environment
		{
		g_print ("%s(): initialize failed\n", __FUNCTION__);
		return CAL_ERR_INITIALIZE;			
		}
  backends = g_ptr_array_new ();
  session_maps =
    g_hash_table_new_full (g_int_hash, g_int_equal, g_free, g_free);

// Load all possible backends in 'backend' 
	ret = _cal_load_backends_bk ();   
	
  return ret;
}

void
_cal_finalize (void)
{
  int i;
  int count;

  g_print ("%s():\n", __FUNCTION__);
  
  _env_finalize ();
  g_hash_table_destroy (session_maps);

  count = backends->len;
  for (i = 0; i < count; i++)
  {
    CalBackend *bk = NULL;

    bk = g_ptr_array_remove_index_fast (backends, 0);
    if (bk->funcs)
    {
      bk->funcs->cal_finalize (bk);
    }

    cal_bk_free (bk);
  }

  g_ptr_array_free (backends, TRUE);

  return;
}


/* Return a session id used by the application to connect to a specific backend */
cal_err_t
cal_open_session (uint32_t index, cal_sid_t *sid)
{
  CalBackend *bk = NULL;  // Internal structure representing a specific backend
  guint32 *svalue = NULL; // Session value
  guint32 *ivalue = NULL; // Index value
  cal_err_t err; //Used later to return some errors
  
  if (sid == NULL)  // Check SID validity. The "sid" shouldn't be NULL.
  {
    return CAL_ERR_INVALID;
  }

  if (session_count == 0) // If this is the first time we open a session
	{
		err = _cal_initialize ();

    if (err != CAL_ERR_NONE)
			{
				return CAL_ERR_INITIALIZE;
			}
		session_count++; // Session correctly opened
	}
		
	/* Check given index is a valid backend index */
	if (index < 0 || index >= backends->len) 
  {
    g_print ("%s(): [%d] is out of range [0 ... %d].\n", __FUNCTION__, index,
             (backends->len - 1));
    return CAL_ERR_OUT_OF_RANGE ;
  }
	// Retrieve backend from the "backends" array
  bk = (CalBackend *) g_ptr_array_index (backends, index);
	// Calculation of the session ID = backend index * SESSION_RANGE + backend reference
  *sid = index *SESSION_RANGE + bk->ref;

  bk->ref++; // A backend can be connected many times with differrent session id

  svalue = g_new0 (guint32, 1);
  *svalue = *sid;
  ivalue = g_new (guint32, 1);
  *ivalue = index;

  g_print ("%s(): sid = %d, index = %d\n", __FUNCTION__, *sid, *ivalue);
	// Add the new session to the session map
  g_hash_table_replace (session_maps, (gpointer) svalue, (gpointer) ivalue);

	switch (bk->type)
    {
      case CAL_TYPE_EVENT:
				switch (bk->stype) // Storage type (database / file)
					{
						case 0: // If database, load the functions and initialize
							cal_funcs_init_db (bk);
								if (bk->funcs)
								{
									if (bk->funcs->cal_initialize (bk) != CAL_ERR_NONE)
									{
										return CAL_ERR_INVALID;
									}
								}
							break;
						case 1: // If file, load the functions and initialize
							break;
						default:
							g_print ("%s(): stype - %d is unsupported.\n", __FUNCTION__, bk->stype);						
					}
        
        break;

      default: // If unknown type
        g_print ("%s(): type [%d] is unsupported.\n", __FUNCTION__, bk->type);
        break;
    }
	
  return CAL_ERR_NONE;
}

cal_err_t
cal_close_session (cal_sid_t sid)
{
  guint32 *index = NULL;
//Retrieve the backend's index from the session_map
  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
  {
		// Retrieve the backend by index from the backends array
    CalBackend *bk =
      (CalBackend *) g_ptr_array_index (backends, *index);

    bk->ref--;
		// Remove the session from the session_maps
    g_hash_table_remove (session_maps, &sid);

  }
	
	session_count--; //Session removed

  if (session_count == 0)  // If no more open sessions
  {
    _cal_finalize();  // Close database
  }


  return CAL_ERR_NONE;
}

cal_err_t cal_get_type (cal_sid_t sid, cal_type_t *type)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
    {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);
    *type = bk->type;   
		
		return CAL_ERR_NONE;			
    }
  return CAL_ERR_INVALID;                                  
}

cal_err_t cal_get_name (cal_sid_t sid, const char **name)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
    {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);
    *name = bk->name;   

		return CAL_ERR_NONE;			
    }

  return CAL_ERR_INVALID;                                  
}

cal_err_t cal_get_prodid (cal_sid_t sid, const char **prodid)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
    {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);
    *prodid = bk->prodid;   

		return CAL_ERR_NONE;			
    }

  return CAL_ERR_INVALID;                                  
}

cal_err_t cal_get_version(cal_sid_t sid, const char **version)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
    {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);
    *version = bk->version;   

		return CAL_ERR_NONE;			
    }

  return CAL_ERR_INVALID;                                  
}

cal_err_t cal_get_calscale(cal_sid_t sid, const char **calscale)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
    {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);
    *calscale = bk->calscale;   

		return CAL_ERR_NONE;			
    }

  return CAL_ERR_INVALID;                                  
}

cal_err_t cal_get_default_timezonid(cal_sid_t sid, const char **timezoneid)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
    {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);
    *timezoneid = bk->defaulttz;   

		return CAL_ERR_NONE;			
    }

  return CAL_ERR_INVALID;                                  
}

cal_err_t cal_add_component (cal_sid_t sid,
                             cal_component_t *component, 
														 char **uid)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
    {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);

    if (bk && bk->funcs)
      {
      return (bk->funcs->cal_add_component (bk, component, uid));
      }
   }
 return CAL_ERR_UNSUPPORTED;                             
}     





cal_err_t cal_get_component_by_id (cal_sid_t sid, const char *uid, const char *rid,
                                   cal_component_t **component)
  
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;


  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
  {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);

    if (bk && bk->funcs)
    {
      return (bk->funcs->cal_get_component_by_id (bk, uid, rid, component));
    }
  }
  return CAL_ERR_UNSUPPORTED;
  
}
                                                                                            
cal_err_t cal_get_components_by_time (cal_sid_t sid,
                                      time_t start,
                                      time_t end,
                                      cal_boolean_t detached,
																		  cal_component_t ***components)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;


  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
  {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);

    if (bk && bk->funcs)
    {
      return (bk->funcs->cal_get_components_by_time (bk, start, end, detached, components));
    }
  }

  return CAL_ERR_UNSUPPORTED;
}

 
cal_err_t cal_remove_component (cal_sid_t sid, const char *uid)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;


  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
  {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);
		
    if (bk && bk->funcs)
    {		
      return (bk->funcs->cal_remove_component (bk, uid));
    }
  }

  return CAL_ERR_UNSUPPORTED;
} 


cal_err_t cal_remove_instance (cal_sid_t sid, const char *uid, const char *rid, 
                               cal_mod_type_t modtype)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;


  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
  {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);
		
    if (bk && bk->funcs)
    {		
      return (bk->funcs->cal_remove_instance (bk, uid, rid, modtype));
    }
  }
}


cal_err_t cal_update_component (cal_sid_t sid, const cal_component_t *component)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;


  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
  {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);
		
    if (bk && bk->funcs)
    {		
      return (bk->funcs->cal_update_component (bk, component));
    }
  }

  return CAL_ERR_UNSUPPORTED;
} 
 
cal_err_t cal_update_instance (cal_sid_t sid, const cal_component_t *instance, 
                                cal_mod_type_t modtype)
{
  CalBackend *bk = NULL;
  guint32 *index = NULL;


  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
  {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);
		
    if (bk && bk->funcs)
    {		
      return (bk->funcs->cal_update_instance (bk, instance, modtype));
    }
  }

  return CAL_ERR_UNSUPPORTED;
} 
 
cal_err_t cal_instance_is_multiple (cal_sid_t sid, const cal_component_t *instance,
                                    cal_boolean_t *bool)
{
	CalBackend *bk = NULL;
  guint32 *index = NULL;


  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);

  if (index)
  {
    bk = (CalBackend *) g_ptr_array_index (backends, *index);
		
    if (bk && bk->funcs)
    {		
      return (bk->funcs->cal_instance_is_multiple (bk, instance, bool));
    }
  }

  return CAL_ERR_UNSUPPORTED;
	
}
 
 
 
 
 
 
 
 
 
 
                               

/* 
vi:ts=2:nowrap:ai:expandtab
*/
