/*	libcalenabler2 - Library of calendaring and scheduling enabler
 *
 *	Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

/**
 * @file 
 *
 * <h3>Calendar error definitions</h3>
 *
 * 
 * @sa http://www.doxygen.org
 * @version 0.1
 * @since 15 Sept 2006
 */

#ifndef	__LIPS_CAL_COMPONENT_PRIV_H
#define	__LIPS_CAL_COMPONENT_PRIV_H


#include  <ical.h>

#include	"caltype.h"
#include	"calcomponent.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */


#define PID_RANGE  4
#define PID_RANGE_FACTOR 15

#define PID_GENERATE(ptype, index) ((ptype << PID_RANGE) + index)
#define PID_GET_PTYPE(pid)     (pid >> PID_RANGE)
#define PID_GET_INDEX(pid)    (pid & PID_RANGE_FACTOR)
  

typedef enum 
	{
		COMP_FLAG_COMPONENT,
		COMP_FLAG_EXISTED_INSTANCE,
		COMP_FLAG_NEW_INSTANCE
	}comp_flag_t;
	
	
/** The component structure, which is a wrapper on icalcomponent.
 *  The "icalcomp" hold real memory. 
 *  In "pid_prop_hash" , the "pid" is the key, "icalproperty" is the value.
 *  The hash table doesn't hold the real property, it only stores the pointers
 *	to the "icalproperty"s in the "icalcomp". The hash table is to enable the
 *	quick seach by "pid".
 *	In "ptype_pid_hsh", the "ptype" is the key, the value is "GArray" of 
 *	"pid"s. The hash table enables quick seach for "pid"s of some certain "ptype".
 *	The "alarms" is a GPtrArray which hold the pointers to the alarm subcomponent
 *  in "icalcomp". 
 */
  struct _cal_component_t
  {
    icalcomponent *icalcomp;

    GHashTable *pid_prop_hash;
    
		GHashTable *ptype_pid_hash;
		
		cal_sid_t sid;

  };


  cal_err_t _cal_component_new_from_icalcomponent (cal_component_t ** comp,
                                               icalcomponent * icalcomp);
		
  cal_err_t cal_component_set_sequence (cal_component_t * cpnt,
                                        uint32_t sequence);
  cal_err_t cal_component_get_sequence (cal_component_t * cpnt,
                                        uint32_t * sequence);
  cal_err_t cal_component_increase_sequence (cal_component_t * cpnt);
  gboolean _cal_component_is_instance (cal_component_t * cpnt);
  char *_cal_component_get_recurid_as_string (cal_component_t * cpnt);
  icalcomponent *cal_component_get_icalcomponent (cal_component_t * cpnt);
  icalproperty *_cal_component_get_prop (cal_component_t * cpnt, uint32_t pid);
  void _scan_icalcomponent (cal_component_t * comp);
	icalproperty* _cal_component_get_property_by_pid (cal_component_t *comp, uint32_t pid);

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_CAL_COMPONENT_PRIV_H */

/*
vi:ts=2:nowrap:ai:expandtab
*/
