 /*	libcalenabler2 - Library of calendaring and scheduling enabler
 *
 *	Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTR
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <string.h>
#include <glib.h>
#include <ical.h>


#include	<calcomponent.h>
#include	<caltype.h>
#include	<calerror.h>

#include  "calcomponent-priv.h"
#include  "calvalue.h"

/*used as a free function to free the "key"s in "ptype_pid_hash" */
static void
pids_free (GArray  *pids) 
{
	g_array_free(pids, TRUE);
  
  return;
}

/*This function is to manipulate the ptype_pid_hash table in a component.
 *It calculate the next pid for a specific ptype */
uint32_t 
_get_next_pid (cal_component_t *comp,  uint32_t ptype)
{
	GArray *pidarray;
	uint32_t pid ;
	
	pidarray = g_hash_table_lookup(comp->ptype_pid_hash, &ptype);
	if (pidarray != NULL)
		{
		pid = g_array_index(pidarray, uint32_t, pidarray->len-1);
		pid ++;
		}
	else 
/*The index is started from 1*/
		pid = PID_GENERATE (ptype, 1);
	
#ifdef DEBUG
	g_print ("%s():pid = %d\n", __FUNCTION__, pid);
#endif	
	return pid;
}

/*This function is to manipulate the ptype_pid_hash table in a component.
 *Given the value of "ptype" and "pid", add them to ptype_pid_hash
 */
void 
_add_elem_to_ptype_pid_hash (cal_component_t *comp,  uint32_t ptype,  uint32_t pid)
{
	GArray *pidarray = NULL;
	

	pidarray = g_hash_table_lookup(comp->ptype_pid_hash, &ptype);
//	g_hash_table_lookup(comp->ptype_pid_hash, temptype);

	if (pidarray == NULL)
  {
		uint32_t * key = NULL;

		pidarray = g_array_new(TRUE, TRUE, sizeof(uint32_t));   
		
    key = g_new0(uint32_t, 1);
    *key = ptype;

    /*
     * insert newly-generated pids array to
     * ptype_pid_hash table.
     */
    g_hash_table_replace(comp->ptype_pid_hash, key, pidarray);
  }
	
	g_array_append_val(pidarray, pid);
}

/*This function is to manipulate the ptype_pid_hash table in a component.
 *Given the value of "ptype" and "pid", remove the "pid" from ptype_pid_hash
 */
cal_err_t 
_remove_elem_from_ptype_pid_hash (cal_component_t *comp,  uint32_t ptype,  uint32_t pid)
{
	GArray *pidarray;
	
	pidarray = g_hash_table_lookup(comp->ptype_pid_hash, &ptype);

	if (pidarray == NULL)
		return CAL_ERR_NOT_FOUND;
	else
  	{
		guint i = 0;
		
		while (i < pidarray->len)
			{
				uint32_t calpid = g_array_index(pidarray, uint32_t, i);
				if (calpid == pid)
					break;
				i++;
			}
		
		if (i == pidarray->len)
			return CAL_ERR_NOT_FOUND;
		
		g_array_remove_index (pidarray,  i);
		
		/*If the properties of ptype has all been removed, the array should be removed from hash table*/
		if (pidarray->len == 0)
			g_hash_table_remove (comp->ptype_pid_hash, &ptype);
			
  }	
	return CAL_ERR_NONE;
}


/*FIXME: don't deal with customized properties*/
cal_err_t cal_component_new (cal_sid_t sid, cal_comp_type_t type, cal_component_t **comp)
{
  icalcomponent *icalcomp;
  icalcomponent_kind kind;

#ifdef DEBUG
	g_print ("%s(): entering, type = %d\n", __FUNCTION__, type);
#endif	
	*comp = g_new0 (cal_component_t , 1);
  if (!(*comp))
  {
		g_print ("%s(): memory allocation error. \n", __FUNCTION__);

    return CAL_ERR_MEM_ALLOC;
  }

  if (type == CAL_NO_TYPE_COMPONENT)
    return CAL_ERR_INVALID;

  /* Figure out the kind and create the icalcomponent */

  switch (type)
  {
  case CAL_EVENT_COMPONENT:
    kind = ICAL_VEVENT_COMPONENT;
    break;

  case CAL_TODO_COMPONENT:
    kind = ICAL_VTODO_COMPONENT;
    break;

  case CAL_ALARM_COMPONENT:
    kind = ICAL_VALARM_COMPONENT;
    break;

  case CAL_JOURNAL_COMPONENT:
    kind = ICAL_VJOURNAL_COMPONENT;
    break;

  case CAL_FREEBUSY_COMPONENT:
    kind = ICAL_VFREEBUSY_COMPONENT;

  case CAL_TIMEZONE_COMPONENT:
    kind = ICAL_VTIMEZONE_COMPONENT;
    break;

  default:
    g_assert_not_reached ();
    kind = ICAL_NO_COMPONENT;
  }

  icalcomp = icalcomponent_new (kind);
  if (!icalcomp)
  {
	 	g_print ("%s(): Could not create the icalcomponent! \n", __FUNCTION__);
    return CAL_ERR_MEM_ALLOC;
  }

  (*comp)->icalcomp = icalcomp;
	
	(*comp)->pid_prop_hash =
    g_hash_table_new_full (g_int_hash, g_int_equal, g_free, NULL);
  
	(*comp)->ptype_pid_hash =
		g_hash_table_new_full (g_int_hash, g_int_equal, g_free, (GDestroyNotify)pids_free);
 #ifdef DEBUG
/*if (	(*comp)->pid_prop_hash == NULL)
	g_print ("%s():%d , the hash table is NULL\n", __FUNCTION__, __LINE__);
else
	g_print ("%s():%d , the hash table is not NULL\n", __FUNCTION__, __LINE__);*/
#endif
   return CAL_ERR_NONE;
}

/* Scans an cal_component_t structure.
 * Check the "icalcomp" for its properties.
 * For each property:
 *   - calculate its pid; add the "pid" and "property" to the "pid_prop_hash"
 *   - add the "ptype" and "pid" to the "ptype_pid_hash"
 */
void
_scan_icalcomponent (cal_component_t *comp)
{
  icalproperty *prop;
	cal_comp_type_t ctype;
	gint i;

	cal_component_get_type (comp, &ctype);
  /* Scan properties */
  for (i = ICAL_ACTION_PROPERTY; i < ICAL_NO_PROPERTY; i++)
  {
    gint count = 0;

    for (prop = icalcomponent_get_first_property (comp->icalcomp, i);
         prop;
         prop =
         icalcomponent_get_next_property (comp->icalcomp, i))
    {
			icalproperty_kind ikind = icalproperty_isa (prop);
      uint32_t *pid = g_new0 (uint32_t, 1);
      uint32_t ptype;
			
			
      count++;
			switch (ikind)
				{
/*string properties*/				
				case ICAL_ATTACH_PROPERTY:
					{
					if (ctype == CAL_ALARM_COMPONENT)
						ptype = CAL_ALARM_ATTACH_PROPERTY;
					else
						ptype = CAL_ATTACHURI_PROPERTY;
					break;
					}		
				case ICAL_CATEGORIES_PROPERTY:
					ptype = CAL_CATEGORIES_PROPERTY;
				break;
				case ICAL_CLASS_PROPERTY:
					ptype = CAL_CLASS_PROPERTY;
				break;
				case ICAL_COMMENT_PROPERTY:
					ptype = CAL_COMMENT_PROPERTY;
				break;
				case ICAL_COMPLETED_PROPERTY:
					ptype = CAL_COMPLETED_PROPERTY;
				break;
				case ICAL_CONTACT_PROPERTY:
					ptype = CAL_CONTACT_PROPERTY;
				break;
				case ICAL_CREATED_PROPERTY:
					ptype = CAL_CREATED_PROPERTY;
				break;
				case ICAL_DESCRIPTION_PROPERTY:
					{
					if (ctype == CAL_ALARM_COMPONENT)
						ptype = CAL_ALARM_DESCRIPTION_PROPERTY;
					else
					ptype = CAL_DESCRIPTION_PROPERTY;
					break;
				}
				case ICAL_DTSTART_PROPERTY:
					ptype = CAL_DTSTART_PROPERTY;
				break;
				case ICAL_DTEND_PROPERTY:
					ptype = CAL_DTEND_PROPERTY;
				break;
				case ICAL_DTSTAMP_PROPERTY:
					ptype = CAL_DTSTAMP_PROPERTY;
				break;
				case ICAL_DUE_PROPERTY:
					ptype = CAL_DUE_PROPERTY;
				break;
				case ICAL_DURATION_PROPERTY:
					ptype = CAL_DURATION_PROPERTY;
				break;
				case ICAL_FREEBUSY_PROPERTY:
					ptype = CAL_FREEBUSY_PROPERTY;
				break;
				case ICAL_LOCATION_PROPERTY:
					ptype = CAL_LOCATION_PROPERTY;
				break;
				case ICAL_METHOD_PROPERTY:
					ptype = CAL_METHOD_PROPERTY;
				break;
				case ICAL_ORGANIZER_PROPERTY:
					ptype = CAL_ORGANIZER_PROPERTY;
				break;
				case ICAL_PERCENTCOMPLETE_PROPERTY:
					ptype = CAL_PERCENTCOMPLETE_PROPERTY;
				break;
				case ICAL_PRIORITY_PROPERTY:
					ptype = CAL_PRIORITY_PROPERTY;
				break;
				case ICAL_RECURRENCEID_PROPERTY:
					ptype = CAL_RECURRENCEID_PROPERTY;
				break;
				case ICAL_EXDATE_PROPERTY:
					ptype = CAL_EXDATE_PROPERTY;
				break;
/*rrule properties*/				
				case ICAL_RRULE_PROPERTY:
					ptype = CAL_RRULE_PROPERTY;
				break;
				case ICAL_EXRULE_PROPERTY:
					ptype = CAL_EXRULE_PROPERTY;
				break;
				case ICAL_SEQUENCE_PROPERTY:
					ptype = CAL_SEQUENCE_PROPERTY;
				break;
				case ICAL_STATUS_PROPERTY:
					ptype = CAL_STATUS_PROPERTY;
				break;
				case ICAL_SUMMARY_PROPERTY:
					ptype = CAL_SUMMARY_PROPERTY;
				break;
				case ICAL_TRANSP_PROPERTY:
					ptype = CAL_TRANSP_PROPERTY;
				break;
				case ICAL_TZID_PROPERTY:
					ptype = CAL_TZID_PROPERTY;
				break;
				case ICAL_TZNAME_PROPERTY:
					ptype = CAL_TZNAME_PROPERTY;
				break;
				case ICAL_TZOFFSETFROM_PROPERTY:
					ptype = CAL_TZOFFSETFROM_PROPERTY;
				break;
				case ICAL_TZOFFSETTO_PROPERTY:
					ptype = CAL_TZOFFSETTO_PROPERTY;
				break;
				case ICAL_TZURL_PROPERTY:
					ptype = CAL_TZURL_PROPERTY;
				break;
				case ICAL_UID_PROPERTY:
					ptype = CAL_UID_PROPERTY;
				break;
				case ICAL_ACTION_PROPERTY:
					ptype = CAL_ALARM_ACTION_PROPERTY;
				break;
				case ICAL_TRIGGER_PROPERTY:
					ptype = CAL_ALARM_TRIGGER_PROPERTY;
				break;
/*integer properties*/
/*To be continued ...*/				
				default:
				/*FIXME: only support a few properties*/
					break;				
				}
      
      *pid = PID_GENERATE (ptype, count);

      g_hash_table_replace (comp->pid_prop_hash, (gpointer) pid, (gpointer) prop);
#ifdef DEBUG
	//g_print ("%s(): %d :pid =%d, %s\n", __FUNCTION__, __LINE__, *pid, icalproperty_as_ical_string(prop) );
#endif		
				
			_add_elem_to_ptype_pid_hash (comp, ptype, *pid);
				
    }
  }

#ifdef DEBUG
	//g_print ("%s(): out\n", __FUNCTION__);
#endif
}

/*give a total copy of the src component*/
cal_err_t
_cal_component_new_clone (const cal_component_t *src, cal_component_t **dest)
{
	cal_comp_type_t type;
	cal_component_t *tempcomp = NULL;
	
	g_return_val_if_fail (src != NULL, CAL_ERR_INVALID);
#ifdef DEBUG
		g_print ("%s():%d \n", __FUNCTION__, __LINE__);
#endif
	cal_component_get_type (src, &type);
	/*FIXME: don't deal with customized properties*/
	cal_component_new (0, type, &tempcomp);
	g_return_val_if_fail (tempcomp != NULL, CAL_ERR_MEM_ALLOC);
	icalcomponent_free (tempcomp->icalcomp);
	tempcomp->icalcomp =  icalcomponent_new_clone (src->icalcomp);
	
#ifdef DEBUG
if (tempcomp->pid_prop_hash == NULL)
	g_print ("%s():%d , the hash table is NULL\n", __FUNCTION__, __LINE__);
else
	g_print ("%s():%d , the hash table is not NULL\n", __FUNCTION__, __LINE__);
#endif
	
	_scan_icalcomponent (tempcomp);
	*dest = tempcomp;

	return CAL_ERR_NONE;
}


/*free a cal_component_t structure:
 *free the icalcomp , the pid_prop_hash, and the ptype_pid_hash.
 */
cal_err_t
cal_component_free (cal_component_t * comp)
{
  g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);

  /* Free the icalcomponent */

  if ((comp->icalcomp != NULL)
      && icalcomponent_get_parent (comp->icalcomp) == NULL)
  {
    icalcomponent_free (comp->icalcomp);
    comp->icalcomp = NULL;
  }

	g_hash_table_destroy (comp->ptype_pid_hash);
  g_hash_table_destroy (comp->pid_prop_hash);
	
  
  g_free (comp);

  return CAL_ERR_NONE;
}

/*Free an array of components*/
cal_err_t
cal_components_free (cal_component_t **comps)
{
	guint i;
	
	for (i =0; comps[i]!=NULL; i++)		
		cal_component_free (comps[i]);		
	g_free (comps);
}

cal_err_t
cal_component_get_type (const cal_component_t *comp, cal_comp_type_t *type)
{
  icalcomponent_kind kind;

  g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);

  kind = icalcomponent_isa (comp->icalcomp);

  switch (kind)
  {
  case ICAL_VEVENT_COMPONENT:
    *type = CAL_EVENT_COMPONENT;
    break;

  case ICAL_VTODO_COMPONENT:
    *type = CAL_TODO_COMPONENT;
    break;

  case ICAL_VALARM_COMPONENT:
    *type = CAL_ALARM_COMPONENT;
    break;

  case ICAL_VJOURNAL_COMPONENT:
    *type = CAL_JOURNAL_COMPONENT;
    break;

  case ICAL_VFREEBUSY_COMPONENT:
    *type = CAL_FREEBUSY_COMPONENT;

  case ICAL_VTIMEZONE_COMPONENT:
    *type = CAL_TIMEZONE_COMPONENT;
    break;

  default:
    g_assert_not_reached ();
    type = CAL_NO_TYPE_COMPONENT;
  }
#ifdef DEBUG 
	g_print ("%s():%d, type =%d\n", __FUNCTION__, __LINE__, *type);
#endif		
  return CAL_ERR_NONE;
}


void list_pids (gpointer  key,  gpointer  value,  gpointer user_data)
{
	GArray *pidarray1, *pidarray2;
	gint i;
	
	pidarray1 = (GArray *) value;
	pidarray2 = (GArray *) user_data;
	
	for (i = 0; i < pidarray1->len; i++)
		{
			uint32_t pid = g_array_index(pidarray1, uint32_t, i);
			g_array_append_val (pidarray2, pid);
		}
}

/*query the ptype_pid_hash to get the "pid"s of the "ptype"*/
cal_err_t 
cal_component_get_properties (const cal_component_t *comp,
                              cal_prop_type_t ptype,
                              uint32_t **pids)
{
	GArray *pidarray = NULL;
	guint i;
	
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
	
	if (ptype == CAL_ANY_PROPERTY)
		{
			pidarray = g_array_new (TRUE, TRUE, sizeof (uint32_t));
			g_hash_table_foreach (comp->ptype_pid_hash, list_pids, pidarray);
		}
	else
	{
		pidarray = g_hash_table_lookup (comp->ptype_pid_hash, &ptype);
		
		if (!pidarray)
			return CAL_ERR_NOT_FOUND;
	}
	
		*pids = g_new0 (uint32_t, (pidarray->len) + 1);
		
	
		for (i = 0 ; i < pidarray->len ; i++)
			{
			(*pids)[i]	= g_array_index(pidarray, uint32_t,i);
			}
	
		(*pids)[i] = PID_INVALID_VALUE;

	
	return CAL_ERR_NONE;
}

/*add a property of the specified type to the component
	This function will check whether the ptype exists in "comp";otherwise, it'll
	return CAL_ERR_INVALID.
*/
cal_err_t 
cal_component_add_property (cal_component_t *comp, 
                            cal_prop_type_t ptype,
                            uint32_t *pid)
{
	icalproperty *prop = NULL;
	guint icalptype;
	uint32_t *calpid ;
	cal_comp_type_t ctype;
	
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
	
	cal_component_get_type (comp, &ctype);	
	
#ifdef DEBUG
	g_print ("%s():pid = %d\n", __FUNCTION__, *calpid);
#endif	
	switch (ptype)
		{
/*string properties*/			
			case CAL_ATTACHURI_PROPERTY:
				icalptype = ICAL_ATTACH_PROPERTY;
			break;			
			case CAL_ATTENDEE_PROPERTY:
				icalptype = ICAL_ATTENDEE_PROPERTY;
			break;
			case CAL_CATEGORIES_PROPERTY:
				icalptype = ICAL_CATEGORIES_PROPERTY;
			break;
			case CAL_CLASS_PROPERTY:
				icalptype = ICAL_CLASS_PROPERTY;
			break;
			case CAL_COMMENT_PROPERTY:
				icalptype = ICAL_COMMENT_PROPERTY;
			break;
			case CAL_COMPLETED_PROPERTY:
				icalptype = ICAL_COMPLETED_PROPERTY;
			break;
			case CAL_CONTACT_PROPERTY:
				icalptype = ICAL_CONTACT_PROPERTY;
			break;
			case CAL_CREATED_PROPERTY:
				icalptype = ICAL_CREATED_PROPERTY;
			break;
			case CAL_DESCRIPTION_PROPERTY:
				icalptype = ICAL_DESCRIPTION_PROPERTY;
			break;
			case CAL_DTSTART_PROPERTY:
				icalptype = ICAL_DTSTART_PROPERTY;
			break;
			case CAL_DTSTAMP_PROPERTY:
				icalptype = ICAL_DTSTAMP_PROPERTY;
			break;
			case CAL_DTEND_PROPERTY:
				icalptype = ICAL_DTEND_PROPERTY;
			break;
			case CAL_DUE_PROPERTY:
				icalptype = ICAL_DUE_PROPERTY;
			break;
			case CAL_DURATION_PROPERTY:
				icalptype = ICAL_DURATION_PROPERTY;
			break;
			case CAL_FREEBUSY_PROPERTY:
				icalptype = ICAL_FREEBUSY_PROPERTY;
			break;
			case CAL_LOCATION_PROPERTY:
				icalptype = ICAL_LOCATION_PROPERTY;
			break;
			case CAL_METHOD_PROPERTY:
				icalptype = ICAL_METHOD_PROPERTY;
			break;
			case CAL_ORGANIZER_PROPERTY:
				icalptype = ICAL_ORGANIZER_PROPERTY;
			break;
			case CAL_PERCENTCOMPLETE_PROPERTY:
				icalptype = ICAL_PERCENTCOMPLETE_PROPERTY;
			break;
			case CAL_PRIORITY_PROPERTY:
				icalptype = ICAL_PRIORITY_PROPERTY;
			break;
			case CAL_STATUS_PROPERTY:
				icalptype = ICAL_STATUS_PROPERTY;
			break;
			case CAL_SUMMARY_PROPERTY:
				icalptype = ICAL_SUMMARY_PROPERTY;
			break; 			
			case CAL_RECURRENCEID_PROPERTY:
				icalptype = ICAL_RECURRENCEID_PROPERTY;
			break;
			case CAL_RRULE_PROPERTY:
				icalptype = ICAL_RRULE_PROPERTY;
			break;	
			case CAL_EXRULE_PROPERTY:
				icalptype = ICAL_EXRULE_PROPERTY;
			break;			
			case CAL_SEQUENCE_PROPERTY:
				icalptype = ICAL_SEQUENCE_PROPERTY;
			break;
			case CAL_TRANSP_PROPERTY:
				icalptype = ICAL_TRANSP_PROPERTY;
			break;
			case CAL_TZID_PROPERTY:
				if (ctype != CAL_TIMEZONE_COMPONENT)
					return CAL_ERR_INVALID;
				else
					icalptype = ICAL_TZID_PROPERTY;
			break;
			case CAL_TZNAME_PROPERTY:
				if (ctype != CAL_TIMEZONE_COMPONENT)
					return CAL_ERR_INVALID;
				else
					icalptype = ICAL_TZNAME_PROPERTY;
				break;
			case CAL_TZOFFSETFROM_PROPERTY:
				if (ctype != CAL_TIMEZONE_COMPONENT)
					return CAL_ERR_INVALID;
				else
					icalptype = ICAL_TZOFFSETFROM_PROPERTY;
			break;
			case CAL_TZOFFSETTO_PROPERTY:
				if (ctype != CAL_TIMEZONE_COMPONENT)
					return CAL_ERR_INVALID;
				else
					icalptype = ICAL_TZOFFSETTO_PROPERTY;
			break;
			case CAL_TZURL_PROPERTY:
				if (ctype != CAL_TIMEZONE_COMPONENT)
					return CAL_ERR_INVALID;
				else
					icalptype = ICAL_TZURL_PROPERTY;
				break;
			case CAL_UID_PROPERTY:
				icalptype = ICAL_UID_PROPERTY;
			break;
			case CAL_EXDATE_PROPERTY:
				icalptype = ICAL_EXDATE_PROPERTY;
			break;
			case CAL_ALARM_ACTION_PROPERTY:
				icalptype = ICAL_ACTION_PROPERTY;
			break;
			case CAL_ALARM_TRIGGER_PROPERTY:
				icalptype = ICAL_TRIGGER_PROPERTY;
			break;
			case CAL_ALARM_DESCRIPTION_PROPERTY:
				icalptype = ICAL_DESCRIPTION_PROPERTY;
			break;
			case CAL_ALARM_ATTACH_PROPERTY:
				icalptype = ICAL_ATTACH_PROPERTY;
			break;
			default:
			break;
		}		
	
	calpid = (uint32_t *)g_malloc0 (sizeof (uint32_t));
	*calpid = _get_next_pid (comp, ptype);		
	prop = icalproperty_new (icalptype);
	icalcomponent_add_property (comp->icalcomp, prop);
		
	g_hash_table_replace (comp->pid_prop_hash, calpid , (gpointer) prop);	
	_add_elem_to_ptype_pid_hash (comp, ptype, *calpid);
		
	
	*pid = *calpid;
		
	return CAL_ERR_NONE;	
}

cal_err_t cal_component_remove_property (cal_component_t *comp,
                                           uint32_t pid)
{
	cal_prop_type_t ptype;
	icalproperty *prop = NULL;
	
	
/*remove the property from icalcomp and pid_prop_hash*/	
	prop = g_hash_table_lookup (comp->pid_prop_hash, &pid);	
	
	if (prop == NULL)
		return CAL_ERR_NOT_FOUND;		
	
	g_hash_table_remove (comp->pid_prop_hash, &pid);
	
	icalcomponent_remove_property (comp->icalcomp,  prop);
	icalproperty_free (prop);
	
/*remove the pid from ptype_pid_hash*/
	ptype = PID_GET_PTYPE (pid);
	
	
	return _remove_elem_from_ptype_pid_hash (comp, ptype, pid);
	//return CAL_ERR_NONE;
}

cal_err_t cal_component_remove_property_by_type (cal_component_t *comp,
                                           cal_prop_type_t ptype)
{
	uint32_t *pids = NULL;
  guint i = 0;

  g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);

  cal_component_get_properties (comp, ptype, &pids);

  if (pids == NULL)
    return CAL_ERR_NOT_FOUND;
  else
    {
    while (pids[i]!=PID_INVALID_VALUE)
		{
			uint32_t pid = pids[i];
      i++;
			cal_component_remove_property (comp, pid);
		}
    
    g_free (pids);

    return CAL_ERR_NONE;
    }
}


/** @brief     A function to get property by pid. It's called in 
* cal_component_set_*_property_value() functions.
* If the input pid is FIRST_PID, remove the properties of the type, and add a 
* new property of the type into the component, then return the new property.
* 
* @param[in]   comp         a pointer to the component.
* @param[in]   pid          property id indicating which property's value to set.
* @param[out]  calpid      newly allocated memory
*
* @return   prop
*/
icalproperty* _cal_component_get_property_by_pid (cal_component_t *comp, uint32_t pid)
{
	icalproperty *prop = NULL;
	cal_prop_type_t ptype;
	
	ptype = PID_GET_PTYPE (pid);

	if (PID_GET_INDEX (pid) != 0)	
		{
			//*calpid = pid;
			prop = g_hash_table_lookup (comp->pid_prop_hash, &pid);									
		}
	else
		{
			/*If property of the type exist, we should firstly remove the properties then add the property*/
	
					uint32_t *pids = NULL;
					guint i = 0;
					uint32_t newpid;
			
					cal_component_get_properties (comp, ptype, &pids);
	
					if (pids != NULL)
					  {
						while (pids[i]!=PID_INVALID_VALUE)
							{
								cal_component_remove_property (comp, pids[i]);	
								i++;
							}
						g_free (pids);
						}	
				
				cal_component_add_property (comp, ptype, &newpid);
				prop = g_hash_table_lookup (comp->pid_prop_hash, &newpid);
				
			}
	
	return prop;
}



cal_err_t _cal_property_set_int32_value (cal_prop_type_t ptype, icalproperty *prop, int32_t value)
{
	switch (ptype)
		{
		case CAL_CLASS_PROPERTY:
			icalproperty_set_class (prop, value);
		break;
		case CAL_METHOD_PROPERTY:
			/*FIXME: check component type */
			icalproperty_set_method (prop, value);
		break;
		case CAL_PERCENTCOMPLETE_PROPERTY:
			icalproperty_set_percentcomplete (prop, value);
		break;
		case CAL_PRIORITY_PROPERTY:
			icalproperty_set_priority (prop, value);
		break;
		case CAL_SEQUENCE_PROPERTY:
			icalproperty_set_sequence (prop, value);
		break;
		case CAL_STATUS_PROPERTY:
			icalproperty_set_status (prop, value);
		break;
		case CAL_TRANSP_PROPERTY:
			icalproperty_set_transp (prop, value);
		break;
		case CAL_TZOFFSETFROM_PROPERTY:
			icalproperty_set_tzoffsetfrom (prop, value);
		break;
		case CAL_TZOFFSETTO_PROPERTY:
			icalproperty_set_tzoffsetto (prop, value);
		break;
		case CAL_ALARM_ACTION_PROPERTY:
			/*FIXME: differrent process to operate on alarm properties*/
			icalproperty_set_action (prop, value);
		break;
		case CAL_ALARM_REPEAT_PROPERTY:
			/*FIXME: differrent process to operate on alarm properties*/
			icalproperty_set_repeat (prop, value);
		break;
		case CAL_ALARM_DURATION_PROPERTY:
			icalproperty_set_duration (prop, icaldurationtype_from_int (value));
		break;
		default:
			return CAL_ERR_INVALID;
		}
	return CAL_ERR_NONE;
}

cal_err_t cal_property_set_int32_value					 (cal_component_t *comp,
                                                  uint32_t pid,  
                                                  int32_t value)
{
	cal_prop_type_t ptype;
	icalproperty *prop = NULL;
	icalvalue *calvalue; 
	
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
	
	ptype = PID_GET_PTYPE (pid);	
	g_return_val_if_fail (ptype != CAL_SEQUENCE_PROPERTY, CAL_ERR_INVALID);
	
	/*Check the validity of pid*/
	if (PID_GET_INDEX (pid) == 0)
		return CAL_ERR_INVALID;
	
	prop = g_hash_table_lookup (comp->pid_prop_hash, &pid);	
	if (prop == NULL)
		return CAL_ERR_NOT_FOUND;
	
	return _cal_property_set_int32_value (ptype, prop, value);

}



cal_err_t _cal_property_set_string_value 						(cal_prop_type_t ptype, 
																								icalproperty *prop, 
																								const char *value)
{
	icalvalue *calvalue; 
	
	switch (ptype)
					{
						case CAL_ATTACHURI_PROPERTY:
							{
							icalattach *attach = icalattach_new_from_url(value);
							icalproperty_set_attach (prop, attach);
							icalattach_unref (attach);						
							break;
							}
						case CAL_ATTENDEE_PROPERTY:
							icalproperty_set_attendee (prop, value);
						break;
						case CAL_CATEGORIES_PROPERTY:
							icalproperty_set_categories (prop, value);
						break;
						case CAL_COMMENT_PROPERTY:
							icalproperty_set_comment (prop, value);
						break;
						case CAL_CONTACT_PROPERTY:
							icalproperty_set_contact (prop, value);
						break;
						case CAL_DURATION_PROPERTY:
							icalproperty_set_duration (prop, icaldurationtype_from_string (value));
						break;
						case CAL_FREEBUSY_PROPERTY:
							icalproperty_set_freebusy (prop, icalperiodtype_from_string (value));
						break;
						case CAL_ORGANIZER_PROPERTY:
							icalproperty_set_organizer (prop, value);
						break;
						case CAL_SUMMARY_PROPERTY:
							icalproperty_set_summary (prop, value);
						break;
						case CAL_LOCATION_PROPERTY:
							icalproperty_set_location (prop, value);
						break; 
						case CAL_DESCRIPTION_PROPERTY:
						case CAL_ALARM_DESCRIPTION_PROPERTY:
							icalproperty_set_description (prop, value);
						break;
						case CAL_COMPLETED_PROPERTY:
						case CAL_CREATED_PROPERTY:
						case CAL_DTSTART_PROPERTY:
						case CAL_DTSTAMP_PROPERTY:
						case CAL_DTEND_PROPERTY:
						case CAL_DUE_PROPERTY:
						case CAL_ALARM_TRIGGER_PROPERTY:
							calvalue = icalvalue_new_datetime (icaltime_from_string(value));
							icalproperty_set_value (prop, calvalue);
						break;
						case CAL_RECURRENCEID_PROPERTY:
							calvalue = icalvalue_new_datetime (icaltime_from_string(value));
							icalproperty_set_value (prop, calvalue);
						break;
						case CAL_UID_PROPERTY:
							icalproperty_set_uid (prop, value);
						break;
						case CAL_EXDATE_PROPERTY:
							calvalue = icalvalue_new_datetime (icaltime_from_string(value));
              icalproperty_set_value (prop, calvalue);
						break;
						case CAL_RRULE_PROPERTY:
						case CAL_EXRULE_PROPERTY:
							calvalue = icalvalue_new_recur (icalrecurrencetype_from_string (value));
							icalproperty_set_value (prop, calvalue);								
							break;
						case CAL_TZID_PROPERTY:
							icalproperty_set_tzid (prop, value);
						break;
						case CAL_TZNAME_PROPERTY:
							icalproperty_set_tzname (prop, value);
						break;
						case CAL_TZURL_PROPERTY:
							icalproperty_set_tzurl (prop, value);
						break;
						case CAL_ALARM_ATTACH_PROPERTY:
							{
							icalattach *attach = icalattach_new_from_url (value);
							icalproperty_set_attach (prop, attach);
							icalattach_unref(attach);
							}
							break;
						default:
							return CAL_ERR_INVALID;
						}		
	return CAL_ERR_NONE;
}
cal_err_t cal_property_set_string_value					 (cal_component_t *comp,
                                                   uint32_t pid,	
                                                   const char *value)
{
	cal_prop_type_t ptype;
	icalproperty *prop = NULL;
	
	
	g_return_val_if_fail (CAL_ERR_INVALID, comp != NULL);
	g_return_val_if_fail (CAL_ERR_INVALID, value != NULL);

#ifdef DEBUG
		g_print ("%s():%d \n", __FUNCTION__, __LINE__);
#endif
	ptype = PID_GET_PTYPE (pid);
	
	/*Check the validity of pid*/
	if (PID_GET_INDEX (pid) == 0)
		return CAL_ERR_INVALID;
	
	prop = g_hash_table_lookup (comp->pid_prop_hash, &pid);	
	if (prop == NULL)
		return CAL_ERR_NOT_FOUND;
	
	return _cal_property_set_string_value (ptype, prop, value);
							
}




cal_err_t
_cal_component_new_from_icalcomponent (cal_component_t ** comp,
                                   		icalcomponent * icalcomp)
{
  icalcomponent_kind kind;
	cal_comp_type_t type;
	
  g_return_val_if_fail (icalcomp != NULL, CAL_ERR_INVALID);

  kind = icalcomponent_isa (icalcomp);

	switch (kind)
	{
		case 	ICAL_VEVENT_COMPONENT:
			type = CAL_EVENT_COMPONENT;
		break;
		case ICAL_VTODO_COMPONENT:
			type = CAL_TODO_COMPONENT;
		break;
		case ICAL_VALARM_COMPONENT:
			type = CAL_ALARM_COMPONENT;
		break;
		case ICAL_VJOURNAL_COMPONENT:
			type = CAL_JOURNAL_COMPONENT;
		break;
		case ICAL_VTIMEZONE_COMPONENT:
			type = CAL_TIMEZONE_COMPONENT;
		break;
		default:
			return CAL_ERR_INVALID;		
	}
	
	/*FIXME: don't deal with customized properties*/
	cal_component_new (0, type, comp);
	icalcomponent_free ((*comp)->icalcomp);
 
	(*comp)->icalcomp = icalcomp;
	
#ifdef DEBUG
	g_print ("%s (): icalcomp = %s\n", __FUNCTION__, icalcomponent_as_ical_string((*comp)->icalcomp));
#endif	
  _scan_icalcomponent (*comp);
  //ensure_mandatory_properties (comp);

  return CAL_ERR_NONE;

}

cal_err_t _cal_property_get_int32_value		(cal_prop_type_t ptype, 
																			icalproperty *prop, 
																			int32_t *value)
{
	switch (ptype)
		{
			case CAL_CLASS_PROPERTY:
			case CAL_METHOD_PROPERTY:
			case CAL_PERCENTCOMPLETE_PROPERTY:
			case CAL_PRIORITY_PROPERTY:
			case CAL_SEQUENCE_PROPERTY:
			case CAL_STATUS_PROPERTY:
			case CAL_TRANSP_PROPERTY:	
			case CAL_TZOFFSETFROM_PROPERTY:
			case CAL_TZOFFSETTO_PROPERTY:	
			case CAL_ALARM_ACTION_PROPERTY:
			case CAL_ALARM_REPEAT_PROPERTY:
			case CAL_ALARM_DURATION_PROPERTY:				
				*value = icalvalue_get_integer (icalproperty_get_value (prop));
			break; 
			default:
				return CAL_ERR_INVALID;
		}
	return CAL_ERR_NONE;
}
cal_err_t cal_property_get_int32_value 							(cal_component_t *comp,
                                                    uint32_t pid,
                                                    int32_t *value)
{
 	icalproperty *prop = NULL;
	cal_prop_type_t ptype;
	
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
	ptype = PID_GET_PTYPE (pid);
	
	if (PID_GET_INDEX (pid) == 0)
		return CAL_ERR_INVALID;
	else		
		{
			prop = g_hash_table_lookup (comp->pid_prop_hash, &pid);				
			
			if (prop == NULL)
				return CAL_ERR_INVALID;	
	
		}

	return _cal_property_get_int32_value(ptype, prop, value);
	
}


void _cal_property_get_string_value		(cal_prop_type_t ptype, 
																			icalproperty *prop, 
																			const char **value)
{
#if 0
	if ((ptype == CAL_DTSTART_PROPERTY) || (ptype == CAL_DTEND_PROPERTY)	||	 
		  (ptype == CAL_COMPLETED_PROPERTY) || (ptype == CAL_DUE_PROPERTY) ||
			(ptype == CAL_EXDATE_PROPERTY ) || (ptype == CAL_RDATE_PROPERTY)	||
			(ptype == CAL_RECURRENCEID_PROPERTY ) || (ptype == CAL_ALARM_TRIGGER_PROPERTY))
		*value = icaltime_as_ical_string (icalvalue_get_datetime (icalproperty_get_value (prop)));	
	
	else if ((ptype == CAL_RRULE_PROPERTY) ||(ptype == CAL_EXRULE_PROPERTY) 
					|| (ptype == CAL_ALARM_ATTACH_PROPERTY))
	{
		*value = icalvalue_as_ical_string(icalproperty_get_value (prop));
	}
	else
	{
#endif		
		//*value = icalvalue_get_string (icalproperty_get_value (prop));
		*value = icalvalue_as_ical_string(icalproperty_get_value (prop));
	//}		
}

cal_err_t cal_property_get_string_value						 (const cal_component_t *comp,
																									 uint32_t pid,	
                                                   const char **value)
{
	icalproperty *prop = NULL;
	cal_prop_type_t ptype;
	
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
	ptype = PID_GET_PTYPE (pid);
	
	if (PID_GET_INDEX (pid) == 0)
		return CAL_ERR_INVALID;
	else	
		{

			prop = g_hash_table_lookup (comp->pid_prop_hash, &pid);				
			
			if (prop == NULL)
				return CAL_ERR_INVALID;	
		}


		_cal_property_get_string_value(ptype, prop, value);
		
		
	return CAL_ERR_NONE;
}



cal_err_t cal_proptype_set_int32_value (cal_component_t *comp,
                                        cal_prop_type_t ptype,  
                                        int32_t value)
{
	uint32_t *pids = NULL;
	guint i = 0;
	uint32_t newpid = PID_INVALID_VALUE;
	icalproperty *prop = NULL;
	cal_err_t err;
	
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
	
	cal_component_get_properties (comp, ptype, &pids);
	
	if (pids != NULL)
		{
		while (pids[i]!=PID_INVALID_VALUE)
			i++;
					
		if (i>1)
			return CAL_ERR_INVALID;		
		else
			{
				uint32_t pid = pids[0];
				cal_component_remove_property (comp, pid);
			}
		g_free (pids);
		}
		
	err = cal_component_add_property (comp, ptype, &newpid);
	if (err == CAL_ERR_NONE)
		{		
		prop = g_hash_table_lookup (comp->pid_prop_hash, &newpid);
		if (prop == NULL)
			return CAL_ERR_NOT_FOUND;
			
		return _cal_property_set_int32_value (ptype, prop, value);
		
		}
	else
		return CAL_ERR_INVALID;
}

cal_err_t cal_proptype_set_string_value (cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          const char *value)
{
	uint32_t *pids = NULL;
	guint i = 0;
	uint32_t newpid = PID_INVALID_VALUE;
	icalproperty *prop = NULL;
	cal_err_t err;
	
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
	
	cal_component_get_properties (comp, ptype, &pids);
	
	if (pids != NULL)
		{
		while (pids[i]!=PID_INVALID_VALUE)
			i++;
					
		if (i>1)
			return CAL_ERR_INVALID;		
		else
			{
				uint32_t pid = pids[0];
				cal_component_remove_property (comp, pid);
			}
		g_free (pids);
		}
		
	err = cal_component_add_property (comp, ptype, &newpid);
	if (err == CAL_ERR_NONE)
	{
		prop = g_hash_table_lookup (comp->pid_prop_hash, &newpid);
			
		if (prop == NULL)
			return CAL_ERR_NOT_FOUND;
		
		return _cal_property_set_string_value (ptype, prop, value);
	}
	else
		return CAL_ERR_INVALID;
}

cal_err_t cal_proptype_get_int32_value (cal_component_t *comp,
                                        cal_prop_type_t ptype,
                                        int32_t *value)
{
	uint32_t *pids = NULL;
	guint i = 0;

	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
	
	cal_component_get_properties (comp, ptype, &pids);
	
	if (pids == NULL)
		return CAL_ERR_NOT_FOUND;
	else
	  {
		while (pids[i]!=PID_INVALID_VALUE)
			i++;
					
		if (i>1)
			return CAL_ERR_INVALID;		
		else
			{
			/*There's one property of "ptype" in the component*/	
			uint32_t pid = pids[0];	
			icalproperty *prop = NULL;
				
			prop = g_hash_table_lookup (comp->pid_prop_hash, &pid);	
				
			if (prop == NULL)
				return CAL_ERR_INVALID;	
		
			_cal_property_get_int32_value(ptype, prop, value);	
			}
		g_free (pids);
		return CAL_ERR_NONE;
		}	
}

cal_err_t cal_proptype_get_string_value (cal_component_t *comp,
                                          cal_prop_type_t ptype,
                                          const char **value)
{
	uint32_t *pids = NULL;
	guint i = 0;

	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
	
	cal_component_get_properties (comp, ptype, &pids);
	
	if (pids == NULL)
		return CAL_ERR_NOT_FOUND;
	else
	  {
		while (pids[i]!=PID_INVALID_VALUE)
			i++;
					
		if (i>1)
			return CAL_ERR_INVALID;		
		else
			{
			/*There's one property of "ptype" in the component*/	
			uint32_t pid = pids[0];	
			icalproperty *prop = NULL;
				
			prop = g_hash_table_lookup (comp->pid_prop_hash, &pid);	
				
			if (prop == NULL)
				return CAL_ERR_INVALID;	
		
			_cal_property_get_string_value(ptype, prop, value);
			}
		g_free (pids);
		return CAL_ERR_NONE;
		}	
}



 cal_err_t cal_property_get_type           (const cal_component_t *comp,
                                             uint32_t pid,  
                                             cal_prop_type_t *ptype,
                                             cal_prop_value_type_t *vtype)
{
	*ptype = PID_GET_PTYPE (pid);
	cal_property_get_valuetype (*ptype , vtype);
}

cal_err_t cal_property_get_valuetype (cal_prop_type_t ptype,
                                      cal_prop_value_type_t *vtype)
{
	*vtype = prop[ptype - 1].vtype;
}

cal_err_t cal_property_get_label       (cal_prop_type_t ptype,
                                        const char **label)
{
	*label = prop[ptype - 1].label;
}

cal_err_t cal_property_get_parameters (const cal_component_t *comp,
                                       uint32_t pid,
                                       cal_parameter_type_t **patypes) 
{
	icalproperty *prop = NULL;
	GArray  *pa = g_array_new (FALSE, FALSE, sizeof(int32_t));
	gint i;
	
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
		
	if (PID_GET_INDEX (pid) == 0)
		return CAL_ERR_INVALID;
	else		
		{
			prop = g_hash_table_lookup (comp->pid_prop_hash, &pid);				
			
			if (prop == NULL)
				return CAL_ERR_INVALID;	
		}
		
	/*test the parameters one by one*/
	for (i  = ICAL_ALTREP_PARAMETER; i < ICAL_X_PARAMETER; i++)
		{
			if (icalproperty_get_first_parameter (prop,  i))
				g_array_append_val(pa, i);
		}	
	if (pa->len > 0)
		{
		cal_parameter_type_t *pas;
		
		pas = g_malloc ((pa->len + 1)*sizeof (cal_parameter_type_t));
			
		for (i = 0; i < pa->len; i++)			 
			pas[i] = 	g_array_index (pa, int32_t, i);
						
		pas[pa->len] = CAL_INVALID_PARAMETER;
			
		*patypes = pas;
		}
		g_array_free (pa, TRUE);
}


cal_err_t cal_parameter_set_value       (const cal_component_t *comp,                                                       
                                         uint32_t pid,
                                         cal_parameter_type_t patype,
                                         const char *value)
{
	icalproperty *prop = NULL;
	icalparameter *param; 
		
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
		
	if (PID_GET_INDEX (pid) == 0)
		return CAL_ERR_INVALID;
	else		
		{
			prop = g_hash_table_lookup (comp->pid_prop_hash, &pid);				
			
			if (prop == NULL)
				return CAL_ERR_INVALID;	
		}
	
	param = icalparameter_new_from_value_string (patype, value);
	
	icalproperty_set_parameter (prop, param);
}
 
cal_err_t cal_parameter_get_value     (const cal_component_t *comp, 
                                       uint32_t pid,
                                       cal_parameter_type_t patype,
                                       const char **value)
{
	icalproperty *prop = NULL;
	icalparameter *param; 
	char *pv;
	
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
		
	if (PID_GET_INDEX (pid) == 0)
		return CAL_ERR_INVALID;
	else		
		{
			prop = g_hash_table_lookup (comp->pid_prop_hash, &pid);				
			
			if (prop == NULL)
				return CAL_ERR_INVALID;	
		}
		
		
	*value = icalproperty_get_parameter_as_string (prop, icalparameter_kind_to_string(patype));
	/*param = icalproperty_get_first_parameter (prop, patype);
	*value = icalparameter_as_ical_string (param);
		
	pv = strchr(*value,'=');
		
	*value = pv+1;*/
}


cal_err_t cal_proptype_get_parameters (const cal_component_t *comp,
                                       cal_prop_type_t ptype,
                                       cal_parameter_type_t **patypes)
{
	uint32_t *pids = NULL;
  guint i = 0;

  g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);

  cal_component_get_properties (comp, ptype, &pids);

  if (pids == NULL)
    return CAL_ERR_NOT_FOUND;
  else
    {
    while (pids[i]!=PID_INVALID_VALUE)
      i++;

    if (i>1)
      return CAL_ERR_INVALID;
    else
      {
      /*There's one property of "ptype" in the component*/
      uint32_t pid = pids[0];
				
			cal_property_get_parameters (comp,  pid, patypes);
      }
    g_free (pids);
    return CAL_ERR_NONE;
    }
}


cal_err_t cal_proptype_parameter_get_value     (const cal_component_t *comp, 
                                         					cal_prop_type_t ptype,
                                         					cal_parameter_type_t patype,
                                         					const char **value)
{
	uint32_t *pids = NULL;
  guint i = 0;

  g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);

  cal_component_get_properties (comp, ptype, &pids);

  if (pids == NULL)
    return CAL_ERR_NOT_FOUND;
  else
    {
    while (pids[i]!=PID_INVALID_VALUE)
      i++;

    if (i>1)
      return CAL_ERR_INVALID;
    else
      {
      /*There's one property of "ptype" in the component*/
      uint32_t pid = pids[0];
				
			cal_parameter_get_value (comp, pid, patype, value);
      }
    g_free (pids);
    return CAL_ERR_NONE;
    }
}


cal_err_t cal_proptype_parameter_set_value       (const cal_component_t *comp,                                                       
                                           					cal_prop_type_t ptype,
                                           					cal_parameter_type_t patype,
                                           					const char *value)
{
	uint32_t *pids = NULL;
  guint i = 0;

  g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);

  cal_component_get_properties (comp, ptype, &pids);

  if (pids == NULL)
    return CAL_ERR_NOT_FOUND;
  else
    {
    while (pids[i]!=PID_INVALID_VALUE)
      i++;

    if (i>1)
      return CAL_ERR_INVALID;
    else
      {
      /*There's one property of "ptype" in the component*/
      uint32_t pid = pids[0];
				
			cal_parameter_set_value (comp, pid, patype, value);
      }
    g_free (pids);
    return CAL_ERR_NONE;
    }
}

cal_err_t cal_component_new_from_string (cal_sid_t sid, const char *string,
                                           cal_component_t **comp)
{
	icalcomponent *icalcomp;
  cal_err_t err;

	icalcomp = icalcomponent_new_from_string(string);
#ifdef DEBUG	
	g_print ("%s()%d -- %s\n", __FUNCTION__, __LINE__, icalcomponent_as_ical_string (icalcomp));
#endif	
	err = _cal_component_new_from_icalcomponent(comp, icalcomp);

	if (err != CAL_ERR_NONE)
		{
		g_print ("%s (): _cal_component_new_from_icalcomponent error!\n", __FUNCTION__);		
		return err;
		}
  return CAL_ERR_NONE;
	
}



cal_err_t cal_component_to_string (cal_sid_t sid, const cal_component_t *comp,
                                     char **string)
{
	char *icalstring = NULL;
  
	icalstring = icalcomponent_as_ical_string(comp->icalcomp);	
	*string = icalstring;
		
	return CAL_ERR_NONE;	
}






cal_err_t cal_component_get_alarm (const cal_component_t *comp,
                                      cal_component_t **alarm)
{
	cal_comp_type_t ctype;
	icalcomponent *subicomp;
	
	cal_component_get_type (comp, &ctype);	
	
	g_print ("%s:%d: ctype =%d\n", __FUNCTION__, __LINE__, ctype);
	if ((ctype != CAL_EVENT_COMPONENT) && (ctype != CAL_TODO_COMPONENT))
		return CAL_ERR_INVALID;
	
	if (icalcomponent_count_components (comp->icalcomp,  ICAL_VALARM_COMPONENT) > 0)
		{
		subicomp = icalcomponent_get_first_component (comp->icalcomp, ICAL_VALARM_COMPONENT);
		_cal_component_new_from_icalcomponent (alarm, subicomp);
			
		return CAL_ERR_NONE;
		}
	else 
		return CAL_ERR_NOT_FOUND;
	 
	return CAL_ERR_NONE;	
}

cal_err_t cal_component_add_alarm  (cal_component_t *comp,
                                    const cal_component_t *alarm)
{
	icalcomponent_add_component (comp->icalcomp, icalcomponent_new_clone (alarm->icalcomp));
}

cal_err_t cal_component_remove_alarm (cal_component_t *comp)
{
	icalcomponent *subicomp = icalcomponent_get_first_component (comp->icalcomp, ICAL_VALARM_COMPONENT);
	guint i;
	guint count = icalcomponent_count_components (comp->icalcomp, ICAL_VALARM_COMPONENT);
	for (i = 0; i < count; i++ )
		{
			icalcomponent_remove_component (comp->icalcomp,  subicomp);
			subicomp = icalcomponent_get_next_component (comp->icalcomp, ICAL_VALARM_COMPONENT);
		}
}
#if 0
cal_err_t
cal_component_get_parameter_value_list (cal_component_t * comp,
                                        uint32_t pid,
                                        cal_parameter_kind_t tpaid,
                                        char ***values)
{
  icalproperty *prop;
  icalparameter *param;
  gint i = 0;

  prop = _cal_component_get_prop (comp, pid);
  for (param = icalproperty_get_first_parameter (prop, tpaid);
       param != 0; param = icalproperty_get_next_parameter (prop, tpaid))
  {
    (*values)[i] = g_strdup (icalparameter_as_ical_string (param));
    i++;
  }
  (*values)[i] = NULL;
}

cal_err_t
cal_component_append_parameter_value (cal_component_t * comp,
                                      uint32_t pid,
                                      uint32_t tpaid, const char *value)
{
  icalparameter *param;
  icalproperty *prop;

  param = icalparameter_new_from_value_string (tpaid, value);
  prop = _cal_component_get_prop (comp, pid);
  icalproperty_add_parameter (prop, param);

}

cal_err_t
cal_component_new_from_string (const char *string, cal_component_t ** comp)
{
  icalcomponent *icalcomp;
  icalcomponent_kind kind;

  g_return_val_if_fail (string != NULL, CAL_ERR_INVALID);

  icalcomp = icalparser_parse_string (string);

  if (!icalcomp)
    return CAL_ERR_INVALID;

  kind = icalcomponent_isa (icalcomp);
  if (!(kind == ICAL_VEVENT_COMPONENT
        || kind == ICAL_VTODO_COMPONENT
        || kind == ICAL_VALARM_COMPONENT
        || kind == ICAL_VJOURNAL_COMPONENT
        || kind == ICAL_VFREEBUSY_COMPONENT
        || kind == ICAL_VTIMEZONE_COMPONENT))
    return CAL_ERR_INVALID;

	*comp = g_new0 (cal_component_t , 1);
  if (!(*comp))
  {
    g_print ("%s(): memory allocation. \n", __FUNCTION__);
    return CAL_ERR_MEM_ALLOC;
  }
  (*comp)->icalcomp = icalcomp;
  //_scan_icalcomponent (comp);

  /* Add missing stuff */

  //ensure_mandatory_properties (comp);
  return CAL_ERR_NONE;
}

cal_err_t
cal_component_as_string (cal_component_t * comp, char **string)
{
  char *buf;

  g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
  g_return_val_if_fail (comp->icalcomp != NULL, CAL_ERR_INVALID);

  /* Ensure that the user has committed the new SEQUENCE */
  //g_return_val_if_fail (priv->need_sequence_inc == FALSE, NULL);

  /* We dup the string; libical owns that memory */

  buf = icalcomponent_as_ical_string (comp->icalcomp);

  if (buf)
    *string = g_strdup (buf);
  else
    *string = NULL;

  return CAL_ERR_NONE;
}

cal_err_t
cal_component_clone (cal_component_t * src_comp, cal_component_t ** dest_comp)
{
  icalcomponent *new_icalcomp;

  g_return_val_if_fail (src_comp != NULL, CAL_ERR_INVALID);
  //g_return_val_if_fail (priv->need_sequence_inc == FALSE, NULL);

	*dest_comp = g_new0 (cal_component_t, 1);
  if (!(*dest_comp))
  {
    g_print ("%s(): memory allocation. \n", __FUNCTION__);
		return CAL_ERR_MEM_ALLOC;
  }

  if (src_comp->icalcomp)
  {
    new_icalcomp = icalcomponent_new_clone (src_comp->icalcomp);
    (*dest_comp)->icalcomp = new_icalcomp;
  }

  return CAL_ERR_NONE;
}


cal_err_t
cal_component_list_components (cal_component_t * comp,
                               cal_component_t *** subcomps)
{
  GSList *list = NULL;
  icalcomponent *icalcomp;
  guint size, i;


  for (icalcomp = icalcomoponent_get_first_component (comp, ICAL_ANY_COMPONENT);
       icalcomp != 0;
       icalcomp = icalcomponent_get_next_component (comp, ICAL_ANY_COMPONENT))
  {
    list = g_slist_append (list, icalcomp);
  }
  size = g_slist_length (list);
  *subcomps = g_new0 (cal_component_t *, size);
  if (!(*subcomps))
  {
    g_print ("%s(): memory allocation. \n", __FUNCTION__);
		return CAL_ERR_MEM_ALLOC;
  }
  for (i = 0; i < size; i++)
  {
    (*subcomps)[i] = (cal_component_t *) g_malloc0 (sizeof (cal_component_t));
    (*subcomps)[i]->icalcomp = g_slist_nth (list, i);
  }
  return CAL_ERR_NONE;
}

cal_err_t
cal_component_add_component (cal_component_t * comp, cal_component_t * subcomp)
{
  icalcomponent_add_component (comp->icalcomp, subcomp->icalcomp);
  return CAL_ERR_NONE;
}

cal_err_t
cal_component_remove_component (cal_component_t * comp,
                                cal_component_t * subcomp)
{
  icalcomponent_remove_component (comp->icalcomp, subcomp->icalcomp);
  return CAL_ERR_NONE;
}

cal_err_t cal_component_list_properties (cal_component_t * comp,
                                         uint32_t ** pids)
{
  gint i, j, count = 0;

  count = icalcomponent_count_properties (comp->icalcomp, ICAL_ANY_PROPERTY);
  if (count == 0)
    return CAL_ERR_INVALID;
  *pids = g_new0 (uint32_t, count);
/**In libical,all properties have a single value, and multi-valued properties are
 * broken down into multiple single valued properties during parsing.
 */
  for (i = ICAL_ACTION_PROPERTY; i < ICAL_NO_PROPERTY; i++)
  {
    gint tempcount = icalcomponent_count_properties (comp->icalcomp, i);
    for (j = 0; j < tempcount; j++)
      (*pids)[j] = i;
  }
}

void resemble_cb(gpointer key, gpointer value, gpointer user_data)
{
	icalcomponent *icalcomp = (icalcomponent *) user_data;
	icalproperty *prop = (icalproperty *)value;
	
	icalcomponent_add_property (icalcomp, prop);
	
}
void
_reassemble_icalcomponent (cal_component_t *comp)
{
	icalcomponent *icalcomp;
	icalproperty *prop;
	gint i;

#ifdef DEBUG
	g_print ("%s(): entering\n", __FUNCTION__);
#endif
	
	icalcomp = icalcomponent_new (icalcomponent_isa (comp->icalcomp));
	g_hash_table_foreach (comp->pid_prop_hash, resemble_cb, icalcomp);
	
	icalcomponent_free (comp->icalcomp);
	comp->icalcomp = icalcomp;
	
}

cal_err_t cal_component_get_time_property_value (const cal_component_t *comp,
																									cal_prop_type_t ptype,
                                                   uint32_t pid,	
                                                   cal_time_t **value)
{
	uint32_t *calpid = g_new0 (uint32_t, 1);
 
	icalproperty *prop = NULL;
	struct icaltimetype itime;
	
	
	g_return_if_fail (comp != NULL);
	

	if (ptype == CAL_ANY_PROPERTY)
		{
		if (pid == PID_INVALID_VALUE)		
			return CAL_ERR_INVALID;
		else 
			{		
			*calpid = pid;
			prop = g_hash_table_lookup (comp->pid_prop_hash, (gpointer)calpid);				
			
			if (prop == NULL)
				return CAL_ERR_INVALID;			
			
			}
		}
	else
		{
		if (pid == PID_INVALID_VALUE)
			{

				
				/*FIXME: if there's already pid of the ptype, how to deal*/
				*calpid = PID_GENERATE(ptype, 0);
				
							
				prop = g_hash_table_lookup (comp->pid_prop_hash, (gpointer)calpid);	
				if (prop == NULL)
				return CAL_ERR_INVALID;
			}			
		else
			{
				
					prop = g_hash_table_lookup (comp->pid_prop_hash, (gpointer)calpid);
					if (prop == NULL)
						return CAL_ERR_INVALID;					
					
			}
		}
		
	itime = icalvalue_get_datetime (icalproperty_get_value (prop));
		
	*value = _cal_time_new_from_icaltimetype (itime);	
	g_free (calpid);
	
	
	return CAL_ERR_NONE;
}

cal_err_t cal_component_set_time_property_value (cal_component_t *comp,
																									cal_prop_type_t ptype,
                                                   uint32_t pid,	
                                                   const cal_time_t *value)
{
	uint32_t *calpid = g_new0 (uint32_t, 1);
	icalproperty *prop = NULL;
	icalvalue *calvalue; 
	
	g_return_if_fail (comp != NULL);
	
	if (ptype == CAL_ANY_PROPERTY)
		{
		if (pid == PID_INVALID_VALUE)		
			return CAL_ERR_INVALID;
		else 
			{		
			*calpid = pid;
			prop = g_hash_table_lookup (comp->pid_prop_hash, (gpointer)calpid);				
			
			if (prop == NULL)
				return CAL_ERR_INVALID;			
			
			}
		}
	else
		{
		if (pid == PID_INVALID_VALUE)
			{
				
				/*FIXME: if there's already pid of the ptype, how to deal*/
				*calpid = PID_GENERATE(ptype, 0);
				
												
				prop = g_hash_table_lookup (comp->pid_prop_hash, (gpointer)calpid);	
				if (prop == NULL)
					{
				
					switch (ptype)
					{
					case CAL_DTSTART_PROPERTY:
						ptype = ICAL_DTSTART_PROPERTY;
					break;
					case CAL_DTEND_PROPERTY:
						ptype = ICAL_DTEND_PROPERTY;
					break;
					default:
					break;
					}
#ifdef DEBUG
					g_print ("%s () : ptype = %d \n", __FUNCTION__, ptype);
#endif							
					prop = icalproperty_new (ptype);
					if (prop == 0)
						{
							g_print ("%s(): memory allocation error. \n", __FUNCTION__);
					
							return CAL_ERR_MEM_ALLOC;
						}		
					}
				else
				  {
						icalcomponent_remove_property (comp->icalcomp, prop);
					}
			}			
		else
			{
				
				if (ptype != PID_GET_PTYPE(pid))	
					return CAL_ERR_INVALID;
				else
					{
					prop = g_hash_table_lookup (comp->pid_prop_hash, (gpointer)calpid);
					if (prop == NULL)
						return CAL_ERR_INVALID;					
					}
			}
		}
	
	calvalue = icalvalue_new_datetime (_cal_time_as_icaltimetype (value));
	icalproperty_set_value (prop, calvalue);
	g_hash_table_replace (comp->pid_prop_hash, (gpointer) calpid, (gpointer) prop);		
	
	icalcomponent_add_property (comp->icalcomp, prop);
	return CAL_ERR_NONE;
}



cal_err_t cal_component_set_rrule_property_value (cal_component_t *comp,
																									uint32_t pid,
                                                  const cal_recur_t *value)
{
	//uint32_t *calpid;
	icalproperty *prop = NULL;
	cal_prop_type_t ptype;
	
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
	
	ptype = PID_GET_PTYPE (pid);
	prop = _cal_component_get_property_by_pid (comp, pid);
	
	struct icalrecurrencetype v;
	_cal_recur_as_icalrecur (value, &v);
	switch (ptype)
	{
		case CAL_RRULE_PROPERTY:
			icalproperty_set_rrule (prop, v);
			break;
		case CAL_EXRULE_PROPERTY:
			icalproperty_set_exrule (prop, v);
			break;
		default:
			break;
	}
	//g_free (calpid);
	return CAL_ERR_NONE;
}
cal_err_t cal_component_get_rrule_property_value (cal_component_t *comp,
																									uint32_t pid,	
                                                  const cal_recur_t **value)
{
	uint32_t *calpid = g_new0 (uint32_t, 1);
 	icalproperty *prop = NULL;
	cal_prop_type_t ptype;
	
	g_return_val_if_fail (comp != NULL, CAL_ERR_INVALID);
	ptype = PID_GET_PTYPE (pid);
	
	if (PID_GET_INDEX (pid) != 0)	
	{
		*calpid = pid;
		prop = g_hash_table_lookup (comp->pid_prop_hash, (gpointer)calpid);				
		if (prop == NULL)
		{
			g_free (calpid);
			return CAL_ERR_NOT_FOUND;	
		}
			
	}
	else
	{
		*calpid = PID_GENERATE(ptype, 1);
										
		prop = g_hash_table_lookup (comp->pid_prop_hash, (gpointer)calpid);		
		if (prop == NULL)
		{
			g_free (calpid);
			*value = NULL;
			return CAL_ERR_INVALID;	
		}
	}
		
	_cal_recur_from_icalrecur (icalvalue_get_recur (icalproperty_get_value (prop)), value);
	
	g_free (calpid);
	
	return CAL_ERR_NONE;
}
#endif
