/*  libcalenabler - Library of LiPS address book enabler
 *
 *This software and associated documentation files (the "Software") 
 *are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *All Rights Reserved. 
 *
 *A copyright license is hereby granted for redistribution and use of 
 *the Software in source and binary forms, with or without modification, 
 *provided that the following conditions are met: 
 *- Redistributions of source code must retain the above copyright notice, 
 *this copyright license and the following disclaimer. 
 *- Redistributions in binary form must reproduce the above copyright 
 *notice, this copyright license and the following disclaimer in the 
 *documentation and/or other materials provided with the distribution. 
 *- Neither the name of LiPS  nor the names of its Members may be used 
 *to endorse or promote products derived from the Software without 
 *specific prior written permission. 
 *
 *A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *distribute any implementation compliant with the any specification adopted 
 *by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *and non-discriminatory terms and conditions and under reciprocity, as 
 *regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdio.h>
#include <stdlib.h>

#include <glib.h>

#define DEFAULT_CAL_RES_PATH  PREFIX "/share/calendar2"
#define DEFAULT_DATAPATH			PREFIX "/share"
#define	DATABASE_PRIFIX				"database"
#define	FILE_PRIFIX				    "file"
#define	COMPONENT_TEMPLATE		"component.templ"
#define METADATA_CONFIG_GROUP "group1"
#define	ADMIN_DATABASE				"admin.db"

static gchar *data_path = NULL;
static gchar *source_prefix_db = NULL;
static gchar *source_prefix_file = NULL;
static gchar *admin_db = NULL;

G_CONST_RETURN gchar *
_env_get_admin_database (void)
{
  if (admin_db == NULL)
  {
    admin_db = g_build_filename (source_prefix_db, ADMIN_DATABASE, NULL);
  }

  return admin_db;
}

G_CONST_RETURN gchar *
_env_get_source_prefix_db (void)
{
  return source_prefix_db;
}

G_CONST_RETURN gchar *
_env_get_source_prefix_file (void)
{
  return source_prefix_file;
}


gboolean
_env_initialize (void)
{
  gchar *path = NULL;

  path = getenv ("CAL_RES_PATH");
  
  if (path == NULL)
  {
    g_print ("CAL_RES_PATH isn't set , use %s\n", DEFAULT_CAL_RES_PATH);
    path = DEFAULT_CAL_RES_PATH; 
  }
  data_path = g_strdup (path);

  source_prefix_db = g_build_filename (data_path, DATABASE_PRIFIX, NULL);
  source_prefix_file = g_build_filename (data_path, FILE_PRIFIX, NULL);

  return TRUE;
}

void
_env_finalize (void)
{
  if (data_path)
  {
    g_free (data_path);
  }

  if (source_prefix_db)
  {
    g_free (source_prefix_db);
  }

  if (source_prefix_file)
  {
    g_free (source_prefix_file);
  }


  if (admin_db)
  {
    g_free (admin_db);
  }

  return;
}

/* 
vi:ts=2:nowrap:ai:expandtab
*/
