/*	libcalenabler2 - Library of calendaring and scheduling enabler
 *
 *	Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include  <ical.h>
#include  <time.h> /* for time() */
#include <glib.h>
#include "calicalutils.h"
#include "calcomponent-priv.h"
#include "cal-db.h"



GPtrArray   *_icalrecur_expand_recurrence_by_time(cal_component_t   *comp,
																				 					time_t       range_start, 
																				 					time_t       range_end)
{
	                                                                                  	
	struct icalrecurrencetype recur; 					// Recurrence struct
	icalrecur_iterator* ritr;									// An iterator on the recurrence
	time_t start_t, end_t, interval;
  struct icaltimetype compstart, compend, next;				
  GPtrArray *gparray =  g_ptr_array_new (); // Array to store results

	recur = icalproperty_get_rrule (icalcomponent_get_first_property(comp->icalcomp, ICAL_RRULE_PROPERTY));
	compstart = icalproperty_get_dtstart (icalcomponent_get_first_property(comp->icalcomp, ICAL_DTSTART_PROPERTY));
	compend = icalproperty_get_dtend(icalcomponent_get_first_property(comp->icalcomp, ICAL_DTEND_PROPERTY));	
	start_t = icaltime_as_timet(compstart);
	end_t = icaltime_as_timet(compend);
	interval = end_t - start_t;
				
#ifdef DEBUG
	g_print ("%s():%d recur = %s, compstart = %s, range_start =%d, range_end = %d\n", 
	__FUNCTION__, __LINE__,icalrecurrencetype_as_string(&recur), icaltime_as_ical_string(compstart),
	range_start, range_end	);
#endif
	ritr = icalrecur_iterator_new(recur,compstart);	// Initialize the iterator with the recur and the start time of the component
    
	next = icalrecur_iterator_next(ritr);	// Set the iterator to next valid time according to recurrence
	
	// Test: there is another valid instance and this instance's time is in the specified range
	// As long as the component start time is between "range_start" and "range_end", the instance must happen in the range
	
	while(!icaltime_is_null_time(next))
	{
		start_t = icaltime_as_timet(next);	
		end_t = start_t + interval;
		if ((start_t <= range_end ) && (end_t >= range_start))
			{	
				time_t *t = (time_t *)g_malloc0 (sizeof(time_t));
				
				*t = start_t;
				// Add this instance if it's in the range of time
		    g_ptr_array_add (gparray, (gpointer) t);
			}
		 next = icalrecur_iterator_next(ritr);	// Go to next instance 
    }

    icalrecur_iterator_free(ritr);
  	
#ifdef DEBUG
	g_print ("%s():%d**********%d\n", __FUNCTION__, __LINE__, gparray->len);
#endif
    return gparray;
}

void print_prop_cb (gpointer key, gpointer value, gpointer user_data)
{
	uint32_t *pid = (uint32_t *) key;
	icalproperty *prop = (icalproperty *)value;
	char *svalue = icalproperty_as_ical_string (prop);
	
	g_print ("pid = %d, prop =%s\n", *pid, svalue);
}

/*Used to debug*/
void _cal_component_print_pid_table (cal_component_t *comp)
{
	if (comp->pid_prop_hash)
		g_hash_table_foreach (comp->pid_prop_hash, print_prop_cb,  NULL);
}

void print_pid_cb (gpointer key, gpointer value, gpointer user_data)
{
	uint32_t *ptype = (uint32_t *) key;
	GArray *pidarray;
	guint i=0;
	
	pidarray = value;
	
	g_print ("%s:%d: pidarraylenth =%d\n", __FUNCTION__, __LINE__, pidarray->len);
	while (i < pidarray->len)
			{
				uint32_t pid = g_array_index(pidarray, uint32_t, i);
				g_print ("%s:%d: ptype=%d, pid=%d\n", __FUNCTION__, __LINE__, *ptype, pid);
				i++;
			}
	
	
}

/*Used to debug*/
void _cal_component_print_ptype_table (cal_component_t *comp)
{
	if (comp->ptype_pid_hash)
		g_hash_table_foreach (comp->ptype_pid_hash, print_pid_cb,  NULL);
}
