#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <stdio.h>
#ifdef HAVE_PTHREAD
	#include <pthread.h>
  static pthread_mutex_t tzid_mutex = PTHREAD_MUTEX_INITIALIZER;
#endif


#include <ical.h>
#include "astime.h"
#include "caltime.h"


//typedef icaltimetype _cal_time_t;
struct _cal_time_t
{
  struct tm *tm;
  cal_boolean_t is_UTC;     /**< 1-> time is in UTC timezone */
  cal_boolean_t is_date;    /**< 1 -> interpret this as date. */
	char *tzid;
};

#define HAVE_PTHREAD

cal_err_t cal_time_get_date (const cal_time_t *caltime, 
	                             uint32_t *year, uint32_t *mon, uint32_t *day)
{
	*year = caltime->tm->tm_year + 1900;
	*mon = caltime->tm->tm_mon + 1;
  *day = caltime->tm->tm_mday ;	
}

cal_err_t cal_time_get_time (const cal_time_t *caltime, 
	                             uint32_t *hour, uint32_t *minute, uint32_t *second)
{
	*hour = caltime->tm->tm_hour;
	*minute = caltime->tm->tm_min;
	*second = caltime->tm->tm_sec;
}

cal_err_t			
cal_time_from_timet	(time_t timet, cal_boolean_t is_date, 
                     cal_boolean_t is_UTC, const char *tzid,
										  cal_time_t **caltime)
{
	cal_time_t *internal_time = g_new0 (cal_time_t, 1);
	struct tm *tm_internal;
			
	g_return_val_if_fail (internal_time != NULL, CAL_ERR_MEM_ALLOC);
	
#ifdef HAVE_PTHREAD
		tm_internal = (struct tm*)g_malloc0 (sizeof (struct tm));
		if (is_UTC == TRUE)
    {
			internal_time->is_UTC = TRUE;
			gmtime_r (&timet, tm_internal);
		}
		else 
		{
			localtime_r (&timet, tm_internal);
		}
#else
		if (is_UTC == TRUE)
    {
			internal_time->is_UTC = TRUE;
			
			tm_internal = gmtime (&timet);
		
		}
		else 
		{
			tm_internal = localtime (&timet);
			internal_time->tzid = g_strdup (tzid);
		}
#endif
	
	internal_time->tm = g_memdup (tm_internal, sizeof (struct tm));
	//memcpy (&(internal_time->tm), tm_internal, sizeof (struct tm));
		
#ifdef HAVE_PTHREAD
		g_free (tm_internal);
#endif
	
	if (is_date == TRUE)
	{
		internal_time->tm->tm_sec = -1;
		internal_time->tm->tm_min = -1;		
		internal_time->tm->tm_hour = -1;
		internal_time->is_date = TRUE;
	}
	
	internal_time->tm->tm_isdst = -1;
	*caltime = internal_time;
	return CAL_ERR_NONE;
}

cal_err_t			
cal_time_as_timet	(const cal_time_t *caltime, time_t *timet)
{
	g_return_val_if_fail (caltime != NULL, CAL_ERR_INVALID);
	g_return_val_if_fail (caltime->is_date == FALSE, CAL_ERR_INVALID);

#if 0	
	#ifdef HAVE_PTHREAD
 		pthread_mutex_lock (&tzid_mutex);
  #endif

	if (caltime->is_UTC == TRUE)
	{
		set_tz ("UTC");
	}
	else 
	{
		set_tz (caltime->zone);
	}		

	struct tm tm_tmp;
	memcpy (&tm_tmp, &(caltime->tm), sizeof (struct tm));
		
	*timet = mktime (&(tm_tmp));
	unset_tz ();

	#ifdef HAVE_PTHREAD
    pthread_mutex_unlock (&tzid_mutex);
	#endif
#endif
	return CAL_ERR_NONE;
}


cal_err_t		  
cal_time_free	(cal_time_t *caltime)
{
	g_return_val_if_fail (caltime != NULL, CAL_ERR_INVALID);
	
	g_free (caltime->tm);
	g_free (caltime->tzid);
	g_free (caltime);
	
	return CAL_ERR_NONE;
}

struct icaltimetype
_cal_time_as_icaltimetype (const cal_time_t *caltime)
{
		
	struct icaltimetype tt;
	
	tt.year = caltime->tm->tm_year + 1900;
	tt.month = caltime->tm->tm_mon + 1;
	tt.day = caltime->tm->tm_mday;
	tt.hour = caltime->tm->tm_hour;
	tt.minute = caltime->tm->tm_min;
	tt.second = caltime->tm->tm_sec;
	
	tt.is_daylight = caltime->tm->tm_isdst;
	tt.is_date = (caltime->is_date == TRUE) ? 1 : 0;
	tt.is_utc = (caltime->is_UTC == TRUE) ? 1 : 0;
	
	/*FIXME: timezone not support*/
	tt.zone = NULL;
	
#ifdef DEBUG
	g_print ("%s: icaltime string:%s\n", __FUNCTION__, icaltime_as_ical_string(tt));
#endif	
	return tt;
}

cal_time_t *
_cal_time_new_from_icaltimetype (struct icaltimetype itime)
{
	cal_time_t *internal_time = g_new0 (cal_time_t, 1);
	
	internal_time->tm = (struct tm*)g_malloc0 (sizeof (struct tm));

	internal_time->tm->tm_year = 	itime.year - 1900;
	internal_time->tm->tm_mon = itime.month - 1;
	internal_time->tm->tm_mday = itime.day;
	internal_time->tm->tm_hour = itime.hour;
	internal_time->tm->tm_min = itime.minute;
	internal_time->tm->tm_sec = itime.second;
	
	internal_time->tm->tm_isdst = itime.is_daylight;
	internal_time->is_date = (itime.is_date == 1) ? TRUE:FALSE;
	internal_time->is_UTC = (itime.is_utc == 1) ? TRUE:FALSE;
	

	return internal_time;
}


#if 0
static void 
set_tz (const char *zone)
{
	if (zone == NULL)
		return;
	char *new_tz = (char*)g_malloc0 (strlen (zone) + 43);
	
	/* FIXME: how to specify a relative path */
	strcpy (new_tz, "TZ=/root/svn_root/libcalenabler2/zoneinfo/");
	strcpy (new_tz + 42, zone);

	putenv (new_tz);
}


static void 
unset_tz (void)
{
	unsetenv ("TZ");
  tzset();
}	


cal_err_t		  
cal_time_new (cal_boolean_t is_date, cal_time_t **caltime)
{
	cal_time_t *internal_time =  g_new0 (cal_time_t , 1);
	g_return_val_if_fail (internal_time != NULL, CAL_ERR_MEM_ALLOC);
	
	if (is_date == TRUE)
	{
		internal_time->tm.tm_sec = -1;
		internal_time->tm.tm_min = -1;		
		internal_time->tm.tm_hour = -1;
		internal_time->is_date = TRUE;
	}
	internal_time->tm.tm_isdst = -1;
		
	*caltime = internal_time;
	return CAL_ERR_NONE;
}









cal_err_t 
cal_time_from_string (const char *str, cal_time_t **caltime)
{
	g_return_val_if_fail (str != NULL, CAL_ERR_INVALID);
	
	cal_time_t *internal_time = (cal_time_t*)g_malloc0 (sizeof (cal_time_t));
	g_return_val_if_fail (internal_time != NULL, CAL_ERR_MEM_ALLOC);
  int size;
  size = strlen(str);

	/* floating time with/without separators */
  if ((size == 15) || (size == 19)) 
	{  
		internal_time->is_UTC = FALSE;
		internal_time->is_date = FALSE;
  } 
	/* UTC time, ends in 'Z' */
	else if ((size == 16) || (size == 20)) 
	{ 
		if ((str[15] != 'Z') && (str[19] != 'Z'))
    goto FAIL;

		internal_time->is_UTC = TRUE;
		internal_time->is_date = FALSE;
  } 
	/* A DATE */
	else if ((size == 8) || (size == 10)) 
	{ 
		internal_time->is_UTC = FALSE;
		internal_time->is_date = TRUE;
  } 
	/* error */
	else 
	{ 
 		goto FAIL;
  }
	
  /* DATE */
  if (internal_time->is_date == TRUE)
	{
		if (size == 10) 
		{  
			char dsep1, dsep2;
      if (sscanf(str, "%04d%c%02d%c%02d", &internal_time->tm.tm_year, &dsep1, 
				                                   &internal_time->tm.tm_mon, &dsep2, 
			                                     &internal_time->tm.tm_mday) < 5)
				goto FAIL;

      if ((dsep1 != '-') || (dsep2 != '-'))
        goto FAIL;
    } 
		else if (sscanf(str,"%04d%02d%02d", &internal_time->tm.tm_year, 
				                                   &internal_time->tm.tm_mon, 
			                                     &internal_time->tm.tm_mday) < 3) 
			goto FAIL;
		
		internal_time->tm.tm_sec = -1;
		internal_time->tm.tm_min = -1;		
		internal_time->tm.tm_hour = -1;
  } 
	/* DATE-TIME */
	else 
	{  
		if (size > 16 ) 
		{  
      char dsep1, dsep2, tsep, tsep1, tsep2;
      if (sscanf(str, "%04d%c%02d%c%02d%c%02d%c%02d%c%02d", 
				               &internal_time->tm.tm_year, &dsep1,
			                 &internal_time->tm.tm_mon, &dsep2,
                       &internal_time->tm.tm_mday, &tsep,
			                 &internal_time->tm.tm_hour, &tsep1, 
			                 &internal_time->tm.tm_min, &tsep2,
			                 &internal_time->tm.tm_sec) < 11)
      	goto FAIL;

  
			if((tsep != 'T') || (dsep1 != '-') || (dsep2 != '-') 
				               || (tsep1 != ':') || (tsep2 != ':'))
      	goto FAIL;

    } 
		else 
		{
			char tsep;
  		if (sscanf(str, "%04d%02d%02d%c%02d%02d%02d", 
				               &internal_time->tm.tm_year, 
			                 &internal_time->tm.tm_mon, 
                       &internal_time->tm.tm_mday, &tsep,
			                 &internal_time->tm.tm_hour,  
			                 &internal_time->tm.tm_min, 
			                 &internal_time->tm.tm_sec) < 7)
      	goto FAIL;

			if(tsep != 'T')
      	goto FAIL;
    }
  }
	
	internal_time->tm.tm_year -= 1900;
	internal_time->tm.tm_mon -= 1;
	internal_time->tm.tm_isdst = -1;
	*caltime = internal_time;
	return CAL_ERR_NONE;

	FAIL:
		return CAL_ERR_INVALID;
}


cal_err_t 
cal_time_as_string (const cal_time_t *caltime, char **str)
{
	
	g_return_val_if_fail (caltime != NULL, CAL_ERR_INVALID);
	g_return_val_if_fail (caltime->is_UTC == TRUE || caltime->zone == NULL,
	                      CAL_ERR_INVALID);
	
	int size = 17;
	char *buf = (char*)g_malloc0 (sizeof (char) * size);
  g_return_val_if_fail (buf != NULL, CAL_ERR_MEM_ALLOC);

  if(caltime->is_date == TRUE)
	{
		snprintf(buf, size, "%04d%02d%02d",
		         caltime->tm.tm_year + 1900,
		         caltime->tm.tm_mon + 1,
		         caltime->tm.tm_mday);
  } 
	else 
	{
  	char* fmt;
  	if(caltime->is_UTC == TRUE)
		{
      fmt = "%04d%02d%02dT%02d%02d%02dZ";
		} 
		else 
		{
      fmt = "%04d%02d%02dT%02d%02d%02d";
  	}
  
		snprintf(buf, size, fmt, caltime->tm.tm_year + 1900, 
		                         caltime->tm.tm_mon + 1,
		         							   caltime->tm.tm_mday,
		                         caltime->tm.tm_hour,
		                         caltime->tm.tm_min,
		                         caltime->tm.tm_sec);
  }
  *str = buf;
  return CAL_ERR_NONE;
}


cal_err_t			
cal_time_adjust (cal_time_t *caltime, int32_t days, int32_t hours, 
								 int32_t minutes, int32_t seconds)
{
  g_return_val_if_fail (caltime != NULL, CAL_ERR_INVALID);
  cal_time_t *internal_time = caltime;

  int second, minute, hour, day;
  int minutes_overflow, hours_overflow, days_overflow = 0, years_overflow;
  uint32_t days_in_month;

  if (internal_time->is_date == FALSE)
  {
    second = internal_time->tm.tm_sec + seconds;
    internal_time->tm.tm_sec = second % 60;
    minutes_overflow = second / 60;
    if (internal_time->tm.tm_sec < 0)
    {
      internal_time->tm.tm_sec += 60;
      minutes_overflow--;
    }


    minute = internal_time->tm.tm_min + minutes + minutes_overflow;
    internal_time->tm.tm_min = minute % 60;
    hours_overflow = minute / 60;
    if (internal_time->tm.tm_min < 0)
    {
      internal_time->tm.tm_min += 60;
      hours_overflow--;
    }


    hour = internal_time->tm.tm_hour + hours + hours_overflow;
    internal_time->tm.tm_hour = hour % 24;
    days_overflow = hour / 24;
    if (internal_time->tm.tm_hour < 0)
    {
      internal_time->tm.tm_hour += 24;
      days_overflow--;
    }
  }
	
	if (internal_time->tm.tm_mon > 11)
	{
		years_overflow = internal_time->tm.tm_mon / 12;
		internal_time->tm.tm_year += years_overflow;
		internal_time->tm.tm_mon = internal_time->tm.tm_mon % 12;
	}
	else if (internal_time->tm.tm_mon < 0)
	{
		years_overflow = (internal_time->tm.tm_mon + 1) / 12 - 1;
		internal_time->tm.tm_year += years_overflow;
		internal_time->tm.tm_mon = (internal_time->tm.tm_mon % 12 + 12) % 12;
	}


	day = internal_time->tm.tm_mday + days + days_overflow;
	if (day > 0)
	{
		for (;;)
		{
			cal_time_days_in_month (internal_time->tm.tm_mon,
																						 internal_time->tm.tm_year+1900,
																						 &days_in_month);
			if (day <= days_in_month)
				break;
			internal_time->tm.tm_mon++;
			if (internal_time->tm.tm_mon > 11)
			{
				internal_time->tm.tm_year++;
				internal_time->tm.tm_mon = 0;
			}
			day -= days_in_month;
		}
	}
	else
	{
		while (day <= 0)
		{
			if (internal_time->tm.tm_mon == 0)
			{
				internal_time->tm.tm_year--;
				internal_time->tm.tm_mon = 11;
			}
			else
			{
				internal_time->tm.tm_mon--;
			}
			cal_time_days_in_month (internal_time->tm.tm_mon,
                              internal_time->tm.tm_year+1900, &days_in_month);

      day += days_in_month;
    }
  }
  internal_time->tm.tm_mday = day;

  return CAL_ERR_NONE;

}


static uint32_t days_in_month_static[12] = {31,28,31,30,31,30,31,31,30,31,30,31};
cal_err_t
cal_time_days_in_month (uint32_t month, uint32_t year, uint32_t *days)
{

  if ((month < 0) || (month > 11))
  return CAL_ERR_INVALID;

	*days = days_in_month_static[month];
  if (month == 1)
  {
    *days += ((year % 4==0) && (year % 100 !=0 )) || (year % 400 == 0);
  }

  return CAL_ERR_NONE;
}


cal_err_t			
cal_time_week_number (const cal_time_t *caltime, uint32_t *num)
{
	g_return_val_if_fail (caltime != NULL, CAL_ERR_INVALID);
  
	UTinstant jt;
  memset(&jt, 0, sizeof (UTinstant));

  jt.year = caltime->tm.tm_year + 1900;
	jt.month =caltime->tm.tm_mon + 1;
	jt.day = caltime->tm.tm_mday;
	jt.i_hour = 0;
	jt.i_minute = 0;
	jt.i_second = 0;

  juldat (&jt);
  caldat (&jt);

  *num = (jt.day_of_year - jt.weekday) / 7;
	return CAL_ERR_NONE;
}


cal_err_t			
cal_time_convert_to_zone (cal_time_t *caltime, const	char *zone)
{
	g_return_val_if_fail (caltime != NULL, CAL_ERR_INVALID);
	g_return_val_if_fail (caltime->is_date == FALSE, CAL_ERR_INVALID);
	cal_time_t *internal_time = caltime;
	
	if ((internal_time->is_UTC == FALSE) && (internal_time->zone == zone))
		return CAL_ERR_NONE;
	
	time_t timet_tmp;
	cal_time_as_timet (internal_time, &timet_tmp);


	struct tm *tm_internal;
	#ifdef HAVE_PTHREAD
		tm_internal = (struct tm*)g_malloc0 (sizeof (struct tm));
  	pthread_mutex_lock (&tzid_mutex);
	#endif

	set_tz (zone);
		
	#ifdef HAVE_PTHREAD
		localtime_r (&timet_tmp, tm_internal);		
	#else
		tm_internal = localtime (&timet_tmp);
	#endif

	unset_tz ();

	#ifdef HAVE_PTHREAD
    pthread_mutex_unlock (&tzid_mutex);
	#endif
		
	memcpy (&(internal_time->tm), tm_internal, sizeof (struct tm));
	#ifdef HAVE_PTHREAD
		g_free (tm_internal);
	#endif

	internal_time->is_UTC = FALSE;
	internal_time->zone = zone;
	
	return CAL_ERR_NONE;
}

#endif
